﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

using FieldApp.Extensibility;
using MapHelpers;
using QueryAddin;

namespace ElectricAddin
{

	/// <summary>
	/// Interaction logic for ucLocateFacility.xaml
	/// </summary>
	public partial class ucLocateFacility : UserControl
	{

		private Map _Map;
		private IParentWindow _pParent;
		private string _sLayer;
		private int _iInstance;
		private string _sQuery;
		private FeatureSet _fsResults;
		private int _iFeatureIndex;

		public ucLocateFacility()
		{
			InitializeComponent();
			btnSubmit.Click += btnSubmit_Click;
			btnNext.Click += btnNext_Click;
			btnQuit.Click += btnQuit_Click;
		}

		public void Initialize(Map MyMap, IParentWindow pParent)
		{
			_Map = MyMap;
			_pParent = pParent;
		}

		#region Control Handlers
		private void btnSubmit_Click(object sender, RoutedEventArgs e)
		{

			_fsResults = null;

			// Get query values

			string sID, sFacType;

			sID = tbID.Text;
			if (sID == "")
			{
				MessageBox.Show("Mandatory field not populated.");
				return;
			}
			sFacType = cbFacility.SelectionBoxItem as string;
			if (sFacType == "")
			{
				MessageBox.Show("Select facility type.");
				return;
			}

			// Get parameters

			_sLayer = sFacType;
			_iInstance = 1;
			string sFieldName = "FACILITYID";
			if (sFacType == "Transformer Bank")
				sFieldName = "STRUCTURENUMBER";
			if (sFacType == "Vault")
				_iInstance = 2; // Use the second layer named "Vault" [Because UndergroundStructureAnno has a class called Vault]
			// _sQuery = "TRIM(" + sFieldName + ") = '" + sID + "'";
			//sID = sID.Replace('*', '%');
			//_sQuery = "TRIM(" + sFieldName + ") ";
			//if (sID.Contains('%'))
			//   _sQuery += "LIKE '";
			//else
			//   _sQuery += "= '";
			//_sQuery += sID + "'";
			sID = sID.Replace("*", "");
			_sQuery = sFieldName + " LIKE '%" + sID + "%'";
			this.ExecuteLayerQuery();

		}

		private void btnNext_Click(object sender, RoutedEventArgs e)
		{
			if (_fsResults == null)
				return;
			if (_iFeatureIndex == _fsResults.Count() - 1)
			{
				MessageBox.Show("No more records.");
				return;
			}
			_iFeatureIndex += 1;
			Graphic g = _fsResults.ElementAt(_iFeatureIndex);
			clsQueryHelper.Zoom(g.Geometry, true);
		}

		private void btnQuit_Click(object sender, RoutedEventArgs e)
		{
			_pParent.HideWindow();
		}
		#endregion

		#region Query Handlers
		private void LayerQueryHandler(object sender, QueryEventArgs args)
		{
			FeatureSet fs = args.FeatureSet;
			if (fs.Count() == 0)
			{
				MessageBox.Show("No match found.");
				return;
			}
			_fsResults = fs;
			_iFeatureIndex = 0;
			// Zoom and highlight current feature
			Graphic g = _fsResults.ElementAt(_iFeatureIndex);
			clsQueryHelper.Zoom(g.Geometry, true);
		}

		private void LayerQueryFailed(Object sender, TaskFailedEventArgs args)
		{
			MessageBox.Show("Query failed.");
		}
		#endregion

		private void ExecuteLayerQuery()
		{

			clsCommonHelper ch = clsHelperCollection.CommonHelper;
			string sURL = ch.FindLayerURL(_Map, _sLayer, _iInstance);

			// Get general town limit

			QueryTask qt = new QueryTask(sURL);
			qt.ExecuteCompleted += LayerQueryHandler;
			qt.Failed += LayerQueryFailed;
			Query q = new Query();
			q.Where = _sQuery;
			q.ReturnGeometry = true;
			q.OutSpatialReference = _Map.SpatialReference;
			qt.ExecuteAsync(q);

		}
	
	}

}
