# ProtectiveTrace.py

import arcpy
# Script arguments and variables
sWorkspace = arcpy.GetParameterAsText(0)
# C:/apps/FieldEdit/Data/AZElectric.gdb
if sWorkspace == "":
    arcpy.AddMessage("Returning null result")
    quit()
sNetwork = sWorkspace + "/ElectricDataset/ElectricDataset_Net"
Flags = arcpy.GetParameterAsText(1)
iOutParameter = 2
StopList = ["SwitchBank", "FuseBank", "DynamicProtectiveDeviceBank"]
try:
    arcpy.env.workspace = "in_memory"
    for bNoTraceEnds in [True, False]:
        if bNoTraceEnds:
            LayerList = ["PrimaryOHLineSection", "PrimaryUGLineSection", "TransformerBank"]
            EdgeList = [True, True, False]
            sTraceOpt = "NO_TRACE_ENDS"
        else:
            LayerList = StopList
            EdgeList = [False, False, False]
            sTraceOpt = "TRACE_ENDS"
        # Perform trace
        arcpy.AddMessage("Tracing...")
        Trace_Results = arcpy.CreateUniqueName(sTraceOpt)
        arcpy.TraceGeometricNetwork_management(
            sNetwork, Trace_Results, Flags, "FIND_CONNECTED", "", "", "", "",
            StopList, sTraceOpt)
        # Extract feature sets from group layer
        for iLayer in range(0, len(LayerList)):
            sLayer = LayerList[iLayer]
            bEdge = EdgeList[iLayer]
            arcpy.AddMessage("Processing results (" + sLayer + ")...")
            SelectResults = arcpy.SelectData_management(Trace_Results, sLayer)
            SelectLayer = SelectResults.getOutput(0)
            TempName = arcpy.CreateUniqueName(sLayer, "in_memory")
            arcpy.CopyFeatures_management(SelectLayer, TempName)
            if bEdge:
                # 10.2 or higher: Densify edges to remove any curves
                arcpy.Densify_edit(TempName, "DISTANCE", "10 Feet")
                # 10.1 Workaround: export features to shapefile
##                TempShpName = arcpy.CreateUniqueName(sLayer + ".shp", arcpy.env.scratchFolder)
##                arcpy.CopyFeatures_management(SelectLayer, TempShpName)
##                arcpy.CopyFeatures_management(TempShpName, TempName)
##                arcpy.Delete_management(TempShpName)
            OutFeatureSet = arcpy.FeatureSet()
            OutFeatureSet.load(TempName)
            arcpy.SetParameter(iOutParameter, OutFeatureSet)
            iOutParameter += 1
except Exception, ErrorDesc:
    sErr = "ERROR:\n" + str(ErrorDesc)
    arcpy.AddError(sErr)
