﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.FeatureService;

using FieldApp.Extensibility;

namespace EditAddin
{
	public class EditBehavior : IBehavior, IConfigurable
	{

		public static EditBehavior Current { get; private set; }
		public bool IsInitialized { get; private set; }
		public static clsFieldHelper FieldHelper;
		public static string EditLog;
		public static string LocalLayerID;
		public static string LocalFeatureLayerUrl;
		public static string RemoteFeatureServiceUrl;
		public static ArcGISDynamicMapServiceLayer LocalLayer;
		public static List<string> EditLayerNames;
		public static Dictionary<string, FeatureLayerInfo> EditLayerDetails;

		public void LoadConfiguration(string sConfigData)
		{

			XDocument xDoc = XDocument.Parse(sConfigData);
			XElement xTop = xDoc.Root;
			foreach (XElement xElement in xTop.Elements())
			{
				string sName = xElement.Name.LocalName;
				switch (sName)
				{
					case "EditLog":
						EditLog = xElement.Attribute("Path").Value;
						break;
					case "LocalFeatureLayer":
						LocalLayerID = xElement.Attribute("ID").Value;
						break;
					case "RemoteFeatureService":
						RemoteFeatureServiceUrl = xElement.Attribute("URL").Value;
						break;
					case "EditLayers":
						EditLayerNames = new List<string>();
						EditLayerDetails = new Dictionary<string, FeatureLayerInfo>();
						foreach (XElement xEditLayer in xElement.Elements())
						{
							string sLayerName = xEditLayer.Attribute("Name").Value;
							EditLayerNames.Add(sLayerName);
							EditLayerDetails[sLayerName] = null;
						}
						break;
				}
			}
			
		}

		public void OnAttached()
		{
			Current = this;
			IsInitialized = false;
			Map MyMap = MapApplication.Current.Map;
			LayerCollection MyLayers = MyMap.Layers;
			MyLayers.LayersInitialized += new LayerCollection.LayersInitializedHandler(MyLayers_LayersInitialized);
		}

		public event EventHandler<EventArgs> Initialized;

		void MyLayers_LayersInitialized(object sender, EventArgs args)
		{

			Map MyMap = MapApplication.Current.Map;
			foreach (Layer l in MyMap.Layers)
			{
				if (l.ID != LocalLayerID)
					continue;
				if (!(l is ArcGISDynamicMapServiceLayer))
					break; // This should not happen
				LocalLayer = (ArcGISDynamicMapServiceLayer)l;
				string sMapUrl = LocalLayer.Url;
				LocalFeatureLayerUrl = sMapUrl.Replace("MapServer", "FeatureServer");
				LocalLayer.GetAllDetails(this.AddLayers);
				break;
			}
			
		}

		private void AddLayers(IDictionary<int, FeatureLayerInfo> details, Exception ex)
		{

			if (ex != null)
			{
				string sMsg = "Error getting service details:\n" + ex.Message;
				MessageBox.Show(sMsg, "Edit Behavior", MessageBoxButton.OK);
				return;
			}

			// Populate edit layer details

			foreach (int i in details.Keys)
			{
				FeatureLayerInfo info = details[i];
				string sLayerName = info.Name;
				if (EditLayerDetails.ContainsKey(sLayerName))
					EditLayerDetails[sLayerName] = info;
			}

			// Initialize field helper

			FieldHelper = new clsFieldHelper();
			IStatus pStatus = MapApplication.Current;
			FieldHelper.Initialize(RemoteFeatureServiceUrl, LocalFeatureLayerUrl, EditLayerNames, EditLog, pStatus);
			IsInitialized = true;
			if (this.Initialized != null)
				this.Initialized(this, new EventArgs());

		}

	}
}
