﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

using ESRI.ArcGIS.Client.Geometry;

namespace EditAddin
{
	public class clsFeatureHelper
	{

		public static XElement CreateXElement(Geometry g, IDictionary<string, object> attrs)
		{
			XElement xFeat = new XElement("feature");
			xFeat.SetAttributeValue("type", "object");
			if (g != null)
				xFeat.Add(GetXGeometry(g));
			xFeat.Add(GetXAttributes(attrs));
			return xFeat;
		}

		private static XElement GetXGeometry(Geometry g)
		{
			XElement xGeom = new XElement("geometry");
			xGeom.SetAttributeValue("type", "object");
			if (g is MapPoint)
			{
				MapPoint pt = (MapPoint)g;
				XElement xX = new XElement("x");
				xX.SetAttributeValue("type", "number");
				xX.Value = pt.X.ToString();
				xGeom.Add(xX);
				XElement xY = new XElement("y");
				xY.SetAttributeValue("type", "number");
				xY.Value = pt.Y.ToString();
				xGeom.Add(xY);
			}
			else if (g is Envelope)
			{
				Envelope env = (Envelope)g;
				PointCollection PC = new PointCollection();
				PC.Add(new MapPoint(env.XMin, env.YMin));
				PC.Add(new MapPoint(env.XMin, env.YMax));
				PC.Add(new MapPoint(env.XMax, env.YMax));
				PC.Add(new MapPoint(env.XMax, env.YMin));
				PC.Add(new MapPoint(env.XMin, env.YMin));
				XElement xParts = new XElement("rings");
				xParts.SetAttributeValue("type", "array");
				XElement xPart = new XElement("item");
				xPart.SetAttributeValue("type", "array");
				foreach (MapPoint pt in PC)
				{
					XElement xPt = new XElement("item");
					xPt.SetAttributeValue("type", "array");
					XElement xX = new XElement("item");
					xX.SetAttributeValue("type", "number");
					xX.Value = pt.X.ToString();
					xPt.Add(xX);
					XElement xY = new XElement("item");
					xY.SetAttributeValue("type", "number");
					xY.Value = pt.Y.ToString();
					xPt.Add(xY);
					xPart.Add(xPt);
				}
				xParts.Add(xPart);
				xGeom.Add(xParts);
			}
			else
			{
				string sName = "";
				ObservableCollection<PointCollection> PCList = null;
				if (g is Polyline)
				{
					Polyline p = (Polyline)g;
					sName = "paths";
					PCList = p.Paths;
				}
				else if (g is Polygon)
				{
					Polygon p = (Polygon)g;
					sName = "rings";
					PCList = p.Rings;
				}
				else
					return xGeom;
				XElement xParts = new XElement(sName);
				xParts.SetAttributeValue("type", "array");
				foreach (PointCollection PC in PCList)
				{
					XElement xPart = new XElement("item");
					xPart.SetAttributeValue("type", "array");
					foreach (MapPoint pt in PC)
					{
						XElement xPt = new XElement("item");
						xPt.SetAttributeValue("type", "array");
						XElement xX = new XElement("item");
						xX.SetAttributeValue("type", "number");
						xX.Value = pt.X.ToString();
						xPt.Add(xX);
						XElement xY = new XElement("item");
						xY.SetAttributeValue("type", "number");
						xY.Value = pt.Y.ToString();
						xPt.Add(xY);
						xPart.Add(xPt);
					}
					xParts.Add(xPart);
				}
				xGeom.Add(xParts);
			}
			return xGeom;
		}

		private static XElement GetXAttributes(IDictionary<string, object> attrs)
		{
			XElement xAttr = new XElement("attributes");
			xAttr.SetAttributeValue("type", "object");
			foreach (string sKey in attrs.Keys)
			{
				object oVal = attrs[sKey];
				if (oVal == null)
				{
					XElement xNullElement = new XElement(sKey);
					xNullElement.SetAttributeValue("type", "null");
					xAttr.Add(xNullElement);
					continue;
				}
				string sVal = oVal.ToString();
				string sType = "string";
				if ((oVal is int) || (oVal is double))
					sType = "number";
				else if (oVal is DateTime)
				{
					// Convert to JSON value
					DateTime dt = (DateTime)oVal;
					DateTime epoch = new DateTime(1970, 1, 1, 0, 0, 0, 0);
					TimeSpan span = dt - epoch;
					double dm = span.TotalMilliseconds;
					sVal = dm.ToString("#");
					sType = "number";
				}
				XElement xElement = new XElement(sKey);
				xElement.SetAttributeValue("type", sType);
				xElement.Value = sVal;
				xAttr.Add(xElement);
			}
			return xAttr;
		}


	}
}
