﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Net;
using System.Runtime.Serialization.Json;
using System.Web;
using System.Xml;
using System.Xml.Linq;

namespace EditAddin
{

	public abstract class clsRESTService
	{

		protected string _sBaseURL = "";

		protected XDocument REST_GetResponseAsXml(string sURL)
		{

			// Create request

			HttpWebRequest r = (HttpWebRequest)WebRequest.Create(sURL);
			r.UseDefaultCredentials = true;
			r.PreAuthenticate = true;
			r.Credentials = CredentialCache.DefaultCredentials;

			// Get results and convert JSON to XML

			HttpWebResponse result = (HttpWebResponse)r.GetResponse();
			string sResult;
			using (StreamReader sr = new StreamReader(result.GetResponseStream()))
			{
				sResult = sr.ReadToEnd();
			}
			byte[] byteData = UTF8Encoding.UTF8.GetBytes(sResult);
			XmlDictionaryReader xr = System.Runtime.Serialization.Json.JsonReaderWriterFactory.CreateJsonReader(byteData, XmlDictionaryReaderQuotas.Max);
			XDocument xDoc = XDocument.Load(xr);
			return xDoc;

		}

		protected bool REST_PostRequest(string sURL, byte[] byteData)
		{

			// Set up request

			int iLen = byteData.Length;
			HttpWebRequest r = (HttpWebRequest)WebRequest.Create(sURL);
			r.Method = "POST";
			r.ContentType = "application/x-www-form-urlencoded";
			r.UseDefaultCredentials = true;
			r.PreAuthenticate = true;
			r.Credentials = CredentialCache.DefaultCredentials;
			r.ContentLength = iLen;
			using (Stream s = r.GetRequestStream())
			{
				s.Write(byteData, 0, iLen);
			}

			// Get results

			HttpWebResponse result = (HttpWebResponse)r.GetResponse();
			string sResult;
			using (StreamReader sr = new StreamReader(result.GetResponseStream()))
			{
				sResult = sr.ReadToEnd();
			}
			return (sResult.IndexOf("error") == -1);

		}

		protected string GetLayerUrl(String sLayerName)
		{
			if (_sBaseURL == "")
				return "";
			int iLayerID = this.FindServiceLayer(_sBaseURL, sLayerName);
			if (iLayerID == -1)
				return "";
			return _sBaseURL + "/" + iLayerID.ToString();
		}

		protected string Feature2JSON(XElement xInFeat, int iObjectID, bool bSendToServer)
		{

			// Get and process attributes element

			XElement xInGeometry = null;
			XElement xInAttributes = null;
			XElement xOutFeat = new XElement("root"); // This name is necessary to convert to JSON
			xOutFeat.SetAttributeValue("type", "object");
			XElement xOutAttributes = new XElement("attributes");
			xOutAttributes.SetAttributeValue("type", "object");
			foreach (XElement xChild in xInFeat.Elements())
			{
				string sName = xChild.Name.LocalName;
				if (sName == "geometry")
					xInGeometry = xChild;
				if (sName == "attributes")
					xInAttributes = xChild;
			}
			if (xInAttributes == null)
				return ""; // This should never happen
			if (xInGeometry != null)
				xOutFeat.Add(xInGeometry);

			// Insert OID if an update

			bool bUpdate = (iObjectID > 0);
			if (bUpdate)
			{
				XElement xOID = new XElement("OBJECTID");
				xOID.SetAttributeValue("type", "number");
				xOID.Value = iObjectID.ToString();
				xOutAttributes.Add(xOID);
			}

			// Process remaining attributes

			foreach (XElement xAttr in xInAttributes.Elements())
			{

				string sName = xAttr.Name.LocalName.ToUpper();

				// Weed out source-specific attributes
				if (sName == "OBJECTID")
					continue;
				if (sName == "GLOBALID")
					continue;

				// The timestamp field of the feature to send to the remote server is updated to the current time.
				// Otherwise, given time1 < time2 < time3,
				// if a field user creates or updates a feature at time1,
				// but doesn't synchronize until time3,
				// another field user that synchronizes at time2 will never see the update.
				// The downside to this approach is that a user will in turn receive self updates
				// in which only the timestamp changed, which makes the process a little chattier,
				// but still much better than receiving and checking every single feature for changes.

				if ((sName == "TIMESTAMP") && bSendToServer)
				{
					DateTime dt = DateTime.Now;
					DateTime epoch = new DateTime(1970, 1, 1, 0, 0, 0, 0);
					TimeSpan span = dt - epoch;
					double dm = span.TotalMilliseconds;
					xAttr.Value = dm.ToString("#");
				}
				xOutAttributes.Add(xAttr);
			}
			xOutFeat.Add(xOutAttributes);

			// Now convert to JSON

			XmlReader xr = xOutFeat.CreateReader();
			string sJSON;
			using (MemoryStream ms = new MemoryStream())
			{
				using (XmlDictionaryWriter xw = JsonReaderWriterFactory.CreateJsonWriter(ms, UTF8Encoding.UTF8))
				{
					xw.WriteNode(xr, true);
				}
				sJSON = UTF8Encoding.UTF8.GetString(ms.ToArray());
			}
			return sJSON;

		}

		private int FindServiceLayer(string sServiceURL, string sLayerName)
		{

			// Get JSON service description and read into XML document

			string sURL = sServiceURL + "?f=json";
			XDocument xDoc = this.REST_GetResponseAsXml(sURL);

			// Search through layers and tables

			XElement xRoot = xDoc.Root;
			List<XNode> SearchList = new List<XNode>();
			foreach (XElement xChild in xRoot.Elements())
			{
				string sName = xChild.Name.LocalName;
				if ((sName == "layers") || (sName == "tables"))
					SearchList.Add(xChild);
			}
			foreach (XElement xArray in SearchList)
			{
				foreach (XElement xLayer in xArray.Nodes())
				{
					string sID = "-1";
					string sName = "";
					foreach (XElement xItem in xLayer.Nodes())
					{
						if (xItem.Name.LocalName == "id")
							sID = xItem.Value;
						if (xItem.Name.LocalName == "name")
							sName = xItem.Value;
					}
					if (sName.ToLower() == sLayerName.ToLower())
						return Convert.ToInt32(sID);
				}
			}
			return -1;

		}


	}
}
