﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Xml;
using System.Xml.Linq;

using FieldApp.Extensibility;

namespace EditAddin
{

	// SyncFeatureService encapsulates a feature service for synchronization that is not replica-aware

	class clsSyncFeatureService : clsRESTService, ISynchronize
	{

		private const int iQueryCount = 10; // Number of objectids to query at a time

		private bool _bLocal = false;
		private List<string> _LayerList;
		private IStatus _pStatus = null;

		public clsSyncFeatureService(bool bLocal, string sServiceURL, List<string> LayerList)
		{
			//string sLocal = "http://127.0.0.1:";
			//int i = sLocal.Length;
			//_bLocal = (sServiceURL.Substring(0, i).ToLower() == sLocal);
			_bLocal = bLocal;
			string sURL = sServiceURL;
			if (sURL.IndexOf("/FeatureServer") == -1)
				sURL += "/FeatureServer";
			_sBaseURL = sURL;
			_LayerList = LayerList;
		}

		#region ISynchronize Implementation
		public void SetStatus(IStatus pStatus)
		{
			_pStatus = pStatus;
		}

		public int GetUpdates(Dictionary<string, ActionSet> Actions, ISynchronize pTarget)
		{
			int iResult = 0;
			try
			{
				iResult = this.ProcessActions(Actions, pTarget);
			}
			catch (Exception ex)
			{
				string sMsg = "Could not process actions:\n" + ex.Message + "\n" + ex.StackTrace;
				this.Error(sMsg);
				iResult = -1;
			}
			return iResult;
		}

		public int GetUpdates(DateTime dtSync, ISynchronize pTarget)
		{
			int iResult = 0;
			try
			{
				string sWhereClause;
				if (dtSync == DateTime.MinValue)
					sWhereClause = "1 = 1";
				else
				{
					// 10.2+: Use standardized query syntax
					sWhereClause = "TIMESTAMP >= timestamp '" + dtSync.ToString("yyyy-MM-dd HH:mm:ss") + "'";
					// 10.1: Use Oracle syntax for Oracle-based feature service
					//sWhereClause = "TIMESTAMP >= TO_DATE('";
					//sWhereClause += dtSync.ToString("yyyy-MM-dd HH:mm:ss");
					//sWhereClause += "','YYYY-MM-DD HH24:MI:SS')";
				}
				iResult = this.QueryAndProcessUpdates(sWhereClause, pTarget);
			}
			catch (Exception ex)
			{
				string sMsg = "Could not query and process updates:\n" + ex.Message + "\n" + ex.StackTrace;
				this.Error(sMsg);
				iResult = -1;
			}
			return iResult;
		}

		public int ProcessFeatures(Dictionary<string, UpdateSet> Features)
		{
			int iResult = 0;
			try
			{
				foreach (string sLayerName in Features.Keys)
				{
					UpdateSet updateSet = Features[sLayerName];
					for (int i = 0; i < 2; i++)
					{
						List<XElement> FeatList = null;
						bool bAdd;
						if (i == 0)
						{
							FeatList = updateSet.AddFeatureList;
							bAdd = true;
						}
						else
						{
							FeatList = updateSet.UpdateFeatureList;
							bAdd = false;
						}
						if (FeatList == null)
							continue;
						iResult += this.PostFeatures(sLayerName, FeatList, bAdd);
					}
				}
			}
			catch (Exception ex)
			{
				string sMsg = "Could not process features:\n" + ex.Message + "\n" + ex.StackTrace;
				this.Error(sMsg);
				iResult = -1;
			}
			return iResult;
		}
		#endregion

		#region Private Methods
		private void Status(string sMsg)
		{
			if (_pStatus != null)
				_pStatus.Status(sMsg);
		}

		private void Error(string sMsg)
		{
			if (_pStatus != null)
				_pStatus.Error(sMsg);
		}

		private int ProcessActions(Dictionary<string, ActionSet> Actions, ISynchronize pTarget)
		{

			// Retrieve features and process all at once

			if (Actions.Count == 0)
				return 0;
			Dictionary<string, UpdateSet> d = new Dictionary<string, UpdateSet>();
			foreach (string sLayerName in Actions.Keys)
			{

				ActionSet actionSet = Actions[sLayerName];
				this.Status("Querying " + sLayerName + "...");
				UpdateSet updateSet = new UpdateSet();
				for (int j = 0; j < 2; j++)
				{

					// Get objectids

					this.Status("Querying " + sLayerName + "...");
					List<string> SyncList; 
					bool bAdd;
					if (j == 0)
					{
						SyncList = actionSet.AddIDList;
						bAdd = true;
					}
					else
					{
						SyncList = actionSet.UpdateIDList;
						bAdd = false;
					}
					if (SyncList == null)
						continue;
					if (SyncList.Count == 0)
						continue;
					StringBuilder sb = new StringBuilder();
					sb.Append("IDSTAMP IN (");
					bool bFirst = true;
					foreach (string sID in SyncList)
					{
						if (bFirst)
							bFirst = false;
						else
							sb.Append(", ");
						sb.Append("'" + sID + "'");
					}
					sb.Append(" )");
					string sWhereClause = sb.ToString();
					List<int> IDList = this.QueryOIDs(sLayerName, sWhereClause);
					int iIDCount = IDList.Count;
					if (iIDCount == 0)
						continue;

					// Query features 

					this.Status("Receiving " + sLayerName + "...");
					int iFeatCount = 0;
					List<XElement> FeatList = new List<XElement>();
					int iCount = 0;
					List<int> QueryList = new List<int>();
					for (int i = 0; i < iIDCount; i++)
					{
						QueryList.Add(IDList[i]);
						iCount++;
						if ((iCount < iQueryCount) && (i < iIDCount - 1))
							continue;
						iFeatCount += iCount;
						this.Status("Receiving " + sLayerName + "..." + iFeatCount.ToString());
						XContainer xRoot = this.QueryFeatures(sLayerName, QueryList);
						XContainer xSubFeatures = this.GetFeatures(xRoot);
						if (updateSet.FieldInfo == null)
							updateSet.FieldInfo = this.GetFields(xRoot);
						foreach (XElement xFeat in xSubFeatures.Elements())
							FeatList.Add(xFeat);
						QueryList.Clear();
						iCount = 0;
					}
					if (bAdd)
						updateSet.AddFeatureList = FeatList;
					else
						updateSet.UpdateFeatureList = FeatList;

				}
				d[sLayerName] = updateSet;
			}
			this.Status("Processing features...");
			return pTarget.ProcessFeatures(d);

		}
		
		private int QueryAndProcessUpdates(string sWhereClause, ISynchronize pTarget)
		{

			// Assume this is always update rather than add

			int iTotal = 0;
			foreach (string sLayerName in _LayerList)
			{

				// Get objectids

				this.Status("Querying " + sLayerName + "...");
				List<int> IDList = this.QueryOIDs(sLayerName, sWhereClause);
				int iIDCount = IDList.Count;
				if (iIDCount == 0)
					continue;

				// Query and process features iQueryCount at a time

				this.Status("Processing " + sLayerName + "...");
				int iFeatCount = 0;
				int iCount = 0;
				List<int> QueryList = new List<int>();
				for (int i = 0; i < iIDCount; i++)
				{
					QueryList.Add(IDList[i]);
					iCount++;
					if ((iCount < iQueryCount) && (i < iIDCount - 1))
						continue;
					iFeatCount += iCount;
					this.Status("Processing " + sLayerName + "..." + iFeatCount.ToString());
					XContainer xRoot = this.QueryFeatures(sLayerName, QueryList);
					XContainer xFeatures = this.GetFeatures(xRoot);
					List<XElement> FeatList = new List<XElement>();
					foreach (XElement xFeat in xFeatures.Elements())
						FeatList.Add(xFeat);
					UpdateSet updateSet = new UpdateSet();
					updateSet.FieldInfo = this.GetFields(xRoot);
					updateSet.UpdateFeatureList = FeatList;
					Dictionary<string, UpdateSet> d = new Dictionary<string, UpdateSet>();
					d[sLayerName] = updateSet;
					int iResult = pTarget.ProcessFeatures(d);
					if (iResult < 0)
						return iResult;
					iTotal += iResult;
					QueryList.Clear();
					iCount = 0;
				}
			}
			return iTotal;

		}

		private List<int> QueryOIDs(string sLayerName, string sWhereClause)
		{

			// Get URL

			List<int> IDList = new List<int>();
			string sURL = this.GetLayerUrl(sLayerName);
			if (sURL == "")
				return IDList;

			// Build request
			
			StringBuilder sb = new StringBuilder();

			sb.Append(sURL);
			sb.Append("/query?f=json&returnIdsOnly=true&where=");
			sb.Append(HttpUtility.UrlEncode(sWhereClause));
			string sQueryURL = sb.ToString();

			// Get results and build ID list

			XDocument xDoc = this.REST_GetResponseAsXml(sQueryURL);
			XElement xRoot = xDoc.Root;
			foreach (XElement xElement in xRoot.Elements())
			{
				string sName = xElement.Name.LocalName;
				if (sName == "objectIds")
				{
					foreach (XElement xID in xElement.Nodes())
					{
						string sID = xID.Value;
						int iID = Convert.ToInt32(sID);
						IDList.Add(iID);
					}
					break;
				}
			}
			return IDList;

		}

		private XContainer QueryFeatures(string sLayerName, List<int> IDList)
		{

			// Result will include a "fields" element and a "features" element

			string sURL = this.GetLayerUrl(sLayerName);
			if (sURL == "")
				return null;

			// Build request

			StringBuilder sb = new StringBuilder();
			sb.Append(sURL);
			sb.Append("/query?f=json&outFields=*&objectIds=");
			string sSep = HttpUtility.UrlEncode(", ");
			bool bFirst = true;
			foreach (int iID in IDList)
			{
				if (bFirst)
					bFirst = false;
				else
					sb.Append(sSep);
				sb.Append(iID.ToString());
			}
			string sQueryURL = sb.ToString();

			// Get results

			XDocument xDoc = this.REST_GetResponseAsXml(sQueryURL);
			XNode xRoot = xDoc.Root;
			return (XContainer)xRoot;

		}

		XContainer GetFeatures(XContainer xRoot)
		{
			foreach (XElement xParent in xRoot.Nodes())
			{
				string sName = xParent.Name.LocalName;
				if (sName == "features")
					return xParent;
			}
			return new XElement("features"); // This should never happen
		}

		Dictionary<string, string> GetFields(XContainer xRoot)
		{
			Dictionary<string, string> FieldInfo = new Dictionary<string, string>();
			XElement xFieldInfo = null;
			foreach (XElement xParent in xRoot.Nodes())
			{
				string sName = xParent.Name.LocalName;
				if (sName != "fields")
					continue;
				xFieldInfo = xParent;
				break;
			}
			if (xFieldInfo == null)
				return FieldInfo; // This should never happen

			foreach (XElement xField in xFieldInfo.Elements())
			{
				string sName = "";
				string sType = "";
				foreach (XElement xItem in xField.Elements())
				{
					string sItemName = xItem.Name.LocalName;
					if (sItemName == "name")
						sName = xItem.Value;
					if (sItemName == "type")
						sType = xItem.Value;
				}
				if ((sName == "") || (sType == ""))
					continue; // This should never happen
				if (sName == "OBJECTID")
					continue;
				if (sName == "GLOBALID")
					continue;
				// (None of the shape related fields should appear)
				FieldInfo[sName] = sType;
			}
			return FieldInfo;

		}

		private int PostFeatures(string sLayerName, List<XElement> FeatList, bool bAdd)
		{

			// Build feature lookup

			Dictionary<string, XElement> FeatLookup = new Dictionary<string, XElement>();
			foreach (XElement xFeat in FeatList)
			{
				string sID = "";
				foreach (XElement xChild in xFeat.Elements())
				{
					string sName = xChild.Name.LocalName;
					if (sName != "attributes")
						continue;
					foreach (XElement xAttr in xChild.Elements())
					{
						string sAttr = xAttr.Name.LocalName.ToUpper();
						if (sAttr != "IDSTAMP")
							continue;
						sID = xAttr.Value;
						break;
					}
					break;
				}
				if (sID == "")
					continue;
				FeatLookup[sID] = xFeat;
			}
			if (FeatLookup.Count == 0)
				return 0;
			
			// If not marked as adds, get IDs for updates
			// If necessary, separate existing features from nonexisting
			// and treat nonexisting as adds

			Dictionary<string, int> IDList = null;
			Dictionary<string, XElement> AddLookup = new Dictionary<string, XElement>();
			Dictionary<string, XElement> UpdateLookup = new Dictionary<string, XElement>();
			if (bAdd)
				AddLookup = FeatLookup;
			else
			{
				List<string> SyncList = new List<string>();
				foreach (string sID in FeatLookup.Keys)
					SyncList.Add(sID);
				IDList = this.GetIDsForUpdate(sLayerName, SyncList);
				if (IDList.Count == 0)
					AddLookup = FeatLookup;
				else if (IDList.Count == FeatLookup.Count)
					UpdateLookup = FeatLookup;
				else
				{
					foreach (string sID in FeatLookup.Keys)
					{

						if (IDList.ContainsKey(sID))
							UpdateLookup[sID] = FeatLookup[sID];
						else
							AddLookup[sID] = FeatLookup[sID];
					}
				}
			}

			// Send post request(s)

			int iTotal = 0;
			if (AddLookup.Count > 0)
				iTotal += this.SendUpdates(sLayerName, AddLookup, null);
			if (UpdateLookup.Count > 0)
				iTotal += this.SendUpdates(sLayerName, UpdateLookup, IDList);
			return iTotal;

		}

		private Dictionary<string, int> GetIDsForUpdate(string sLayerName, List<string> SyncList)
		{

			// Get objectid for each sync id

			Dictionary<string, int> d = new Dictionary<string, int>();
			foreach (string sID in SyncList)
			{
				string sWhereClause = "IDSTAMP = '" + sID + "'";
				List<int> IDList = this.QueryOIDs(sLayerName, sWhereClause);
				if (IDList == null)
					continue;
				if (IDList.Count == 0)
					continue;
				d[sID] = IDList[0];
			}
			return d;

		}

		private int SendUpdates(string sLayerName, Dictionary<string, XElement> FeatLookup, Dictionary<string, int> IDList)
		{

			// Create JSON feature array

			bool bAdd = (IDList == null);
			bool bSendToServer = (!_bLocal);
			StringBuilder sb = new StringBuilder();
			sb.Append("[ ");
			bool bFirst = true;
			int iCount = 0;
			foreach (string sKey in FeatLookup.Keys)
			{
				int iID = 0;
				if (!bAdd)
					iID = IDList[sKey];
				XElement xFeat = FeatLookup[sKey];
				string sJSON = this.Feature2JSON(xFeat, iID, bSendToServer);
				if (bFirst)
					bFirst = false;
				else
					sb.Append(", ");
				sb.Append(sJSON);
				iCount++;
			}
			if (bFirst)
				return 0; // This should never happen
			sb.Append(" ]");
			string sFeatures = sb.ToString();

			string sRequestURL = this.GetLayerUrl(sLayerName);
			if (bAdd)
				sRequestURL += "/addFeatures";
			else
				sRequestURL += "/updateFeatures";
			StringBuilder data = new StringBuilder();
			data.Append("f=" + HttpUtility.UrlEncode("json"));
			data.Append("&features=" + HttpUtility.UrlEncode(sFeatures));
			byte[] byteData = UTF8Encoding.UTF8.GetBytes(data.ToString());
			if (!this.REST_PostRequest(sRequestURL, byteData))
				return 0;
			return iCount;

		}
		#endregion

	}
}
