﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Threading;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Symbols;

namespace MapHelpers
{

	public class CanvasLeftMidpointOffsetter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			// Convert (double) ActualWidth to (double) Canvas.Left
			double dActualWidth = (double)value;
			return -dActualWidth / 2;
		}

		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			// Meaningless
			return 0;
		}
	}

	public class clsGraphicHelper
	{

		private Color _FlashColor;

		public clsGraphicHelper()
		{

			// Initialize flash color

			_FlashColor = new Color();
			_FlashColor.A = 0xFF;
			_FlashColor.R = 0xFF;  // ArcReader = 0x0
			_FlashColor.G = 0x0;   // ArcReader = 0x64
			_FlashColor.B = 0x0;   // ArcReader = 0x0

		}

		public void FlashGeometry(Map MyMap, ESRI.ArcGIS.Client.Geometry.Geometry g, MapPoint ptCenter)
		{

			// Check input geometries

			Envelope env = MyMap.Extent;
			if (!env.Intersects(g.Extent))
				return;
			bool bFlashLines = false;
			Polyline plFlash = null;
			if (ptCenter != null)
			{
				if (env.Intersects(ptCenter.Extent))
				{
					bFlashLines = true;
					plFlash = this.CrossHair(env, ptCenter);
				}
			}

			// Find topmost graphics layer that is visible
			// and whose renderer does not take precedence

			GraphicsLayer gLayer = null;
			foreach (Layer l in MyMap.Layers)
			{
				if (!(l is GraphicsLayer))
					continue;
				if (!l.Visible)
					continue;
				GraphicsLayer gl = (GraphicsLayer)l;
				if (gl.RendererTakesPrecedence)
					continue;
				gLayer = gl;
			}
			if (gLayer == null)
				return;

			// Create feature flash graphic

			Color c = _FlashColor;
			Brush b = new SolidColorBrush(Colors.Red);
			Symbol symFeat;

			if ((g is MapPoint) || (g is MultiPoint))
			{
				SimpleMarkerSymbol sm = new SimpleMarkerSymbol();
				sm.Color = b;
				sm.Style = SimpleMarkerSymbol.SimpleMarkerStyle.Circle;
				sm.Size = 16;
				symFeat = sm;
			}
			else if (g is Polyline)
				symFeat = new SimpleLineSymbol(c, 4);
			else
			{
				SimpleFillSymbol sf = new SimpleFillSymbol();
				sf.Fill = b;
				sf.BorderBrush = b;
				sf.BorderThickness = 4;
				symFeat = sf;
			}
			Graphic grFeat = new Graphic();
			grFeat.Geometry = g;
			grFeat.Symbol = symFeat;

			// Create flash lines graphic

			Graphic grLines = null;
			if (bFlashLines)
			{
				grLines = new Graphic();
				grLines.Geometry = plFlash;
				grLines.Symbol = new SimpleLineSymbol(c, 2);
			}

			// Add and remove graphic(s)
			// TO DO:  It would be cool to try some real animation

			int iCounter = 0;
			DispatcherTimer timer = new DispatcherTimer();
			timer.Interval = new TimeSpan(0, 0, 0, 0, 100); // 100 ms
			timer.Tick += delegate(object s, EventArgs args)
			{
				bool bStop = false;
				switch (iCounter)
				{
					case 0:
						gLayer.Graphics.Add(grFeat);
						gLayer.Refresh();
						break;
					case 3:
						if (bFlashLines)
						{
							gLayer.Graphics.Add(grLines);
							gLayer.Refresh();
						}
						else
							bStop = true;
						break;
					case 4:
						bStop = true;
						break;

				}
				iCounter++;
				if (bStop)
				{
					gLayer.Graphics.Remove(grFeat);
					if (bFlashLines)
						gLayer.Graphics.Remove(grLines);
					gLayer.Refresh();
					timer.Stop();
				}

			};
			timer.Start();

		}

		private Polyline CrossHair(Envelope env, MapPoint pt)
		{
			ObservableCollection<ESRI.ArcGIS.Client.Geometry.PointCollection> paths = new ObservableCollection<ESRI.ArcGIS.Client.Geometry.PointCollection>();
			ESRI.ArcGIS.Client.Geometry.PointCollection path1 = new ESRI.ArcGIS.Client.Geometry.PointCollection();
			path1.Add(new MapPoint(env.XMin, pt.Y));
			path1.Add(new MapPoint(env.XMax, pt.Y));
			paths.Add(path1);
			ESRI.ArcGIS.Client.Geometry.PointCollection path2 = new ESRI.ArcGIS.Client.Geometry.PointCollection();
			path2.Add(new MapPoint(pt.X, env.YMin));
			path2.Add(new MapPoint(pt.X, env.YMax));
			paths.Add(path2);
			Polyline pLine = new Polyline();
			pLine.Paths = paths;
			pLine.SpatialReference = pt.SpatialReference;
			return pLine;
		}

	}



}
