﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows.Media;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Tasks;

namespace MapHelpers
{

	public class UniqueValuesResultArgs : EventArgs
	{
		private List<string> _vals;
		private object _userToken;
		public UniqueValuesResultArgs()
		{
		}
		public List<string> Values
		{
			set { _vals = value; }
			get { return _vals; }
		}
		public object UserState
		{
			set { _userToken = value; }
			get { return _userToken; }
		}
	}

	public class clsUniqueValuesHelper
	{

		#region Public Methods and Events
		public event EventHandler<UniqueValuesResultArgs> UniqueValuesComplete;
		public event EventHandler<HelperFailedArgs> UniqueValuesFailed;

		public void GetUniqueValues(string sUrl, string sFieldName, string sWhereClause = null, object userToken = null)
		{

			// 10.2: Use QueryTask with ReturnDistinctValues

			string sWhere = "1 = 1";
			if (sWhereClause != null)
				sWhere = sWhereClause;
			Query q = new Query();
			q.Where = sWhere;
			q.ReturnGeometry = false;
			q.ReturnDistinctValues = true;
			q.OutFields.Add(sFieldName);
			//q.OrderByFields.Add(new OrderByField(sFieldName));
			QueryTask qt = new QueryTask(sUrl);
			qt.ExecuteCompleted += new EventHandler<QueryEventArgs>(qt_ExecuteCompleted);
			qt.Failed += new EventHandler<TaskFailedEventArgs>(qt_Failed);
			qt.ExecuteAsync(q, userToken);

			// 10.1: Use GenerateRendererTask to get unique values

			//List<string> FieldList = new List<string>() { sFieldName };
			//ColorRamp colorRamp = new ColorRamp();
			//colorRamp.Algorithm = Algorithm.CIELabAlgorithm;
			//colorRamp.From = Colors.Black;
			//colorRamp.To = Colors.White;
			//ObservableCollection<ColorRamp> ColorRamps = new ObservableCollection<ColorRamp>() { colorRamp };
			//UniqueValueDefinition uvDef = new UniqueValueDefinition();
			//uvDef.Fields = FieldList;
			//uvDef.ColorRamps = ColorRamps;
			//GenerateRendererParameters grParam = new GenerateRendererParameters();
			//grParam.ClassificationDefinition = uvDef;
			//if (sWhereClause != null)
			//	grParam.Where = sWhereClause;
			//GenerateRendererTask grt = new GenerateRendererTask(sUrl);
			//grt.ExecuteCompleted += new EventHandler<GenerateRendererResultEventArgs>(grt_ExecuteCompleted);
			//grt.Failed += new EventHandler<TaskFailedEventArgs>(grt_Failed);
			//grt.ExecuteAsync(grParam, userToken);

		}

		#endregion

		#region Event Handlers
		void qt_ExecuteCompleted(object sender, QueryEventArgs e)
		{
			List<string> UniqueValues = new List<string>();
			foreach (Graphic gr in e.FeatureSet)
			{
				IDictionary<string, object> attr = gr.Attributes;
				string sKey = attr.Keys.ElementAt(0);
				UniqueValues.Add(attr[sKey].ToString());
			}
			UniqueValues.Sort();
			UniqueValuesResultArgs args = new UniqueValuesResultArgs();
			args.Values = UniqueValues;
			args.UserState = e.UserState;
			this.UniqueValuesComplete(this, args);
		}

		void qt_Failed(object sender, TaskFailedEventArgs e)
		{
			HelperFailedArgs args = new HelperFailedArgs();
			args.Error = e.Error;
			args.UserState = e.UserState;
			this.UniqueValuesFailed(this, args);
		}

		//private void grt_ExecuteCompleted(object sender, GenerateRendererResultEventArgs e)
		//{

		//	// Get unique values from renderer

		//	GenerateRendererResult grResult = e.GenerateRendererResult;
		//	UniqueValueRenderer uvRenderer = (UniqueValueRenderer)grResult.Renderer;
		//	List<string> UniqueValues = new List<string>();
		//	foreach (UniqueValueInfo uvInfo in uvRenderer.Infos)
		//	{
		//		if (uvInfo.Value == null)
		//			continue;
		//		UniqueValues.Add(uvInfo.Value.ToString());
		//	}
		//	UniqueValues.Sort();
		//	UniqueValuesResultArgs args = new UniqueValuesResultArgs();
		//	args.Values = UniqueValues;
		//	args.UserState = e.UserState;
		//	this.UniqueValuesComplete(this, args);

		//}

		//private void grt_Failed(object sender, TaskFailedEventArgs e)
		//{
		//	HelperFailedArgs args = new HelperFailedArgs();
		//	args.Error = e.Error;
		//	args.UserState = e.UserState;
		//	this.UniqueValuesFailed(this, args);
		//}
		#endregion

	}
}
