﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Imaging;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using FieldApp.Extensibility;

namespace NavAddin
{

	public class NavBehavior : IBehavior
	{

		public static NavBehavior Current { get; private set; }
		public bool IsInitialized { get; private set; }
		public bool CanZoomPrevious { get; private set; }
		public bool CanZoomNext { get; private set; }

		private List<Envelope> _listExtentHistory = new List<Envelope>();
		private int _iCurrentExtentIndex = -1;
		private bool _bNewExtent = true;

		#region IBehavior Implementation
		public void OnAttached()
		{
			Current = this;
			IsInitialized = false;
			this.Initialize();
		}
		#endregion

		#region Public Methods and Events
		public event EventHandler Initialized;

		public event EventHandler MapExtentUpdated;

		public void ZoomPrevious()
		{
			if (!CanZoomPrevious)
				return;
			if (_iCurrentExtentIndex != 0)
			{
				_iCurrentExtentIndex -= 1;
				CanZoomNext = true;
				if (_iCurrentExtentIndex == 0)
					CanZoomPrevious = false;
				_bNewExtent = false;
				Map MyMap = MapApplication.Current.Map;
				SpatialReference sr = MyMap.SpatialReference;
				Envelope e = _listExtentHistory[_iCurrentExtentIndex].Clone();
				e.SpatialReference = sr;
				MyMap.ZoomTo(e);
			}
		}

		public void ZoomNext()
		{
			if (!CanZoomNext)
				return;
			if (_iCurrentExtentIndex != _listExtentHistory.Count - 1)
			{
				_iCurrentExtentIndex += 1;
				CanZoomPrevious = true;
				if (_iCurrentExtentIndex == _listExtentHistory.Count - 1)
					CanZoomNext = false;
				_bNewExtent = false;
				Map MyMap = MapApplication.Current.Map;
				SpatialReference sr = MyMap.SpatialReference;
				Envelope e = _listExtentHistory[_iCurrentExtentIndex].Clone();
				e.SpatialReference = sr;
				MyMap.ZoomTo(e);
			}
		}
		#endregion

		private void Initialize()
		{
			Map MyMap = MapApplication.Current.Map;
			MyMap.Progress += new EventHandler<ProgressEventArgs>(MyMap_Progress);
			MyMap.ExtentChanged += new EventHandler<ExtentEventArgs>(MyMap_ExtentChanged);
		}

		void MyMap_Progress(object sender, ProgressEventArgs e)
		{
			if (IsInitialized)
				return;
			if (e.Progress < 100)
				return;
			Map MyMap = MapApplication.Current.Map;
			Envelope env = MyMap.Extent.Clone();
			_listExtentHistory.Add(env);
			_iCurrentExtentIndex = 0;
			IsInitialized = true;
			if (this.Initialized != null)
				this.Initialized(this, new EventArgs());
		}

		private void MyMap_ExtentChanged(object sender, ExtentEventArgs e)
		{
			if (_iCurrentExtentIndex == -1)
				return;
			if (_bNewExtent)
			{
				Envelope env = _listExtentHistory[_iCurrentExtentIndex];
				//if (env.Equals(e.NewExtent))
				//   return;
				if ((env.XMin == e.NewExtent.XMin) && (env.YMin == e.NewExtent.YMin) &&
					(env.XMax == e.NewExtent.XMax) && (env.YMax == e.NewExtent.YMax))
					return;
				_iCurrentExtentIndex += 1;
				if (_listExtentHistory.Count - _iCurrentExtentIndex > 0)
					_listExtentHistory.RemoveRange(_iCurrentExtentIndex, (_listExtentHistory.Count - _iCurrentExtentIndex));
				if (CanZoomNext)
					CanZoomNext = false;
				_listExtentHistory.Add(e.NewExtent.Clone());
				if (!CanZoomPrevious)
					CanZoomPrevious = true;
			}
			else
				_bNewExtent = true;
			if (this.MapExtentUpdated != null)
				this.MapExtentUpdated(this, new EventArgs());
		}

	}

}
