﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Symbols;

using FieldApp.Extensibility;

namespace NavAddin
{

	public class ZoomInTool : NavTool
	{

		private Map _Map;
		private Draw _Draw;

		public ZoomInTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.ZoomIn.png";
			_sCursorPath = "NavAddin.Images.ZoomIn.cur";
			_sToolTip = "Zoom In";
		}

		protected override void NavInitialized()
		{
			_Map = MapApplication.Current.Map;
			_Draw = new Draw(_Map);
			_Draw.DrawMode = DrawMode.Rectangle;
			_Draw.IsEnabled = false;
			_Draw.DrawComplete += new EventHandler<DrawEventArgs>(_Draw_DrawComplete);
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

		protected override void NavActivateTool()
		{
			_Draw.IsEnabled = true;
		}

		protected override void NavDeactivateTool()
		{
			_Draw.IsEnabled = false;
		}

		private void _Draw_DrawComplete(object sender, DrawEventArgs e)
		{
			_Map.ZoomTo(e.Geometry);
		}
		
	}

	public class ZoomOutTool : NavTool
	{

		private Map _Map;
		private Draw _Draw;

		public ZoomOutTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.ZoomOut.png";
			_sCursorPath = "NavAddin.Images.ZoomOut.cur";
			_sToolTip = "Zoom Out";
		}

		protected override void NavInitialized()
		{
			_Map = MapApplication.Current.Map;
			_Draw = new Draw(_Map);
			_Draw.DrawMode = DrawMode.Rectangle;
			_Draw.IsEnabled = false;
			_Draw.DrawComplete += new EventHandler<DrawEventArgs>(_Draw_DrawComplete);
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

		protected override void NavActivateTool()
		{
			_Draw.IsEnabled = true;
		}

		protected override void NavDeactivateTool()
		{
			_Draw.IsEnabled = false;
		}

		private void _Draw_DrawComplete(object sender, DrawEventArgs e)
		{
			Envelope currentExtent = _Map.Extent;
			Envelope zoomBoxExtent = (Envelope)e.Geometry;
			MapPoint zoomBoxCenter = zoomBoxExtent.GetCenter();
			double whRatioCurrent = currentExtent.Width / currentExtent.Height;
			double whRatioZoomBox = zoomBoxExtent.Width / zoomBoxExtent.Height;
			Envelope envNew = null;
			if (whRatioZoomBox > whRatioCurrent)
			{
				// Use width
				double mapWidthPixels = _Map.Width;
				double multiplier = currentExtent.Width / zoomBoxExtent.Width;
				double newWidthMapUnits = currentExtent.Width * multiplier;
				envNew = new Envelope(new MapPoint(zoomBoxCenter.X - (newWidthMapUnits / 2), zoomBoxCenter.Y),
											 new MapPoint(zoomBoxCenter.X + (newWidthMapUnits / 2), zoomBoxCenter.Y));
			}
			else
			{
				// Use height
				double mapHeightPixels = _Map.Height;
				double multiplier = currentExtent.Height / zoomBoxExtent.Height;
				double newHeightMapUnits = currentExtent.Height * multiplier;
				envNew = new Envelope(new MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y - (newHeightMapUnits / 2)),
											 new MapPoint(zoomBoxCenter.X, zoomBoxCenter.Y + (newHeightMapUnits / 2)));
			}
			if (envNew != null)
			{
				envNew.SpatialReference = _Map.SpatialReference;
				_Map.ZoomTo(envNew);
			}
		}

	}

	public class ZoomInFixedButton : NavButton
	{

		public ZoomInFixedButton()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.ZoomInFxd.png";
			_sToolTip = "Zoom In Fixed";
		}

		protected override void NavInitialized() { }

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

		protected override void NavExecute()
		{
			Map MyMap = MapApplication.Current.Map;
			MyMap.Zoom(2);
		}

	}

	public class ZoomOutFixedButton : NavButton
	{

		public ZoomOutFixedButton()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.ZoomOutFxd.png";
			_sToolTip = "Zoom Out Fixed";
		}

		protected override void NavInitialized() { }

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

		protected override void NavExecute()
		{
			Map MyMap = MapApplication.Current.Map;
			MyMap.Zoom(0.5);
		}

	}

	public class PanTool : NavTool
	{

		public PanTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.Pan.png";
			_sCursorPath = "NavAddin.Images.Pan.cur";
			_sToolTip = "Pan";
		}

		protected override void NavInitialized() { }

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

		protected override void NavActivateTool() { }

		protected override void NavDeactivateTool() { }

	}

	public class ZoomFullButton : NavButton
	{

		private Envelope _envDefault = null;

		public ZoomFullButton()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.FullExtent.png";
			_sToolTip = "Full Extent";
		}

		protected override void NavInitialized() 
		{
			Map MyMap = MapApplication.Current.Map;
			_envDefault = MyMap.Extent.Clone();
		}

		protected override bool NavEnabled() { return true; }

		protected override void NavExtentUpdated() { }

		protected override void NavExecute()
		{
			if (_envDefault == null)
				return;
			Map MyMap = MapApplication.Current.Map;
			MyMap.ZoomTo(_envDefault);
		}

	}

	public class ZoomPrevButton : NavButton
	{

		public ZoomPrevButton()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.UndoDraw.png";
			_sToolTip = "Zoom Previous";
		}

		protected override void NavInitialized() { }

		protected override bool NavEnabled() 
		{
			NavBehavior nb = NavBehavior.Current;
			return nb.CanZoomPrevious;
		}

		protected override void NavExtentUpdated() 
		{
			this.OnIsEnabledChanged();
		}

		protected override void NavExecute()
		{
			NavBehavior nb = NavBehavior.Current;
			nb.ZoomPrevious();
		}

	}

	public class ZoomNextButton : NavButton
	{

		public ZoomNextButton()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sIconPath = "NavAddin.Images.RedoDraw.png";
			_sToolTip = "Zoom Next";
		}

		protected override void NavInitialized() { }

		protected override bool NavEnabled()
		{
			NavBehavior nb = NavBehavior.Current;
			return nb.CanZoomNext;
		}

		protected override void NavExtentUpdated()
		{
			this.OnIsEnabledChanged();
		}

		protected override void NavExecute()
		{
			NavBehavior nb = NavBehavior.Current;
			nb.ZoomNext();
		}

	}

	public class ScaleComboBox : IMapComboBox
	{

		private bool _bEnabled;
		private string[] _Scales = { "0:0", "1:1,000", "1:10,000", "1:24,000", "1:100,000", "1:250,000" };
		private ObservableCollection<object> _Items = new ObservableCollection<object>();
		private Map _Map;

		public ScaleComboBox()
		{
			_bEnabled = false;
			ObservableCollection<object> Items = new ObservableCollection<object>();
			foreach (string sScale in _Scales)
				_Items.Add(sScale);
			_Map = MapApplication.Current.Map;
			_Map.ExtentChanged += new EventHandler<ExtentEventArgs>(_Map_ExtentChanged);
			NavBehavior Nav = NavBehavior.Current;
			Nav.Initialized += new EventHandler(Nav_Initialized);
		}

		public bool IsEditable
		{
			get { return true; }
		}

		public bool IsEnabled
		{
			get { return _bEnabled; }
		}

		public bool IsReadOnly
		{
			get { return false; }
		}

		public ObservableCollection<object> Items
		{
			get { return _Items; }
		}

		public void OnSelectionChanged(int iIndex)
		{
			string sOldScale = _Items[0].ToString();
			string sNewScale = _Items[iIndex].ToString();
			if (sNewScale == sOldScale)
				return;
			this.ZoomToScale(sNewScale);
		}

		public void OnTextEntered(string sNewText)
		{
			string sOldScale = _Items[0].ToString();
			string sNewScale = sNewText;
			if (sNewScale == sOldScale)
				return;
			this.ZoomToScale(sNewScale);
		}

		public event EventHandler<EventArgs> IsEnabledChanged;

		public event EventHandler<EventArgs> ItemsChanged;

		private void Nav_Initialized(object sender, EventArgs e)
		{
			_bEnabled = true;
			this.IsEnabledChanged(this, new EventArgs());
			this.SetScale();
		}

		void _Map_ExtentChanged(object sender, ExtentEventArgs e)
		{
			this.SetScale();
		}

		private void ZoomToScale(string sNewScale)
		{
			if (sNewScale.Contains(':'))
			{
				string[] Tok = sNewScale.Split(':');
				sNewScale = Tok[1];
			}
			sNewScale = sNewScale.Replace(",", "");
			try
			{
				double dNewScale = Convert.ToDouble(sNewScale);
				double dZoomFactor = _Map.Scale / dNewScale;
				_Map.Zoom(dZoomFactor);
			}
			catch
			{
			}
		}

		private void SetScale()
		{
			string sOldScale = _Items[0].ToString();
			double dNewScale = _Map.Scale;
			string sNewScale = "1:" + dNewScale.ToString("N0");
			if (sNewScale == sOldScale)
				return;
			_Items[0] = sNewScale;
			this.ItemsChanged(this, new EventArgs());
		}

	}

}
