﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Symbols;
using ESRI.ArcGIS.Client.Tasks;

using FieldApp.Extensibility;
using MapHelpers;

namespace QueryAddin
{

	/// <summary>
	/// Interaction logic for ucGoToXY.xaml
	/// </summary>
	public partial class ucGoToXY : UserControl
	{

		private enum EntryOperation
		{
			Pan,
			Zoom,
			Flash,
			Marker,
			MarkerXY,
			Balloon
		}

		private class GoToResult
		{
			public MapPoint ptMap;
			public MapPoint ptDD;
			public string Header;
			public override string ToString() { return Header.ToString(); }
		}
	
		private const string sGraphicField = "ResultID";
		private const string sGraphicValue = "GoToXY";

		private bool _bInitialized = false;
		private Map _Map;
		private GraphicsLayer _ResultsLayer;
		private IParentWindow _pParent;
		private esriUnits _MapUnits;
		private esriUnits _EntryUnits;
		private bool _bDMS = false;
		private clsGeometryHelper _gh;
		private GoToResult _CurrentResult;
		private EntryOperation _Operation;

		public ucGoToXY()
		{
			InitializeComponent();
			// Because the Silverlight toolkit ContextMenu has a mouse tracking bug
			// (actually it's a window ownership issue),
			// users must right-click buttons to invoke the context menu.
			// This is not necessary in WPF
			btnPan.Click += new RoutedEventHandler(btnPan_Click);
			btnZoom.Click += new RoutedEventHandler(btnZoom_Click);
			btnFlash.Click += new RoutedEventHandler(btnFlash_Click);
			btnMarker.Click += new RoutedEventHandler(btnMarker_Click);
			btnMarkerXY.Click += new RoutedEventHandler(btnMarkerXY_Click);
			btnBalloon.Click += new RoutedEventHandler(btnBalloon_Click);
			btnClear.Click += new RoutedEventHandler(btnClear_Click);
			btnResults.Click += new RoutedEventHandler(btnResults_Click);
			btnClearResults.Click += new RoutedEventHandler(btnClearResults_Click);
			btnUnits.Click += new RoutedEventHandler(btnUnits_Click);
			miMapUnits.Click += new RoutedEventHandler(miMapUnits_Click);
			miDD.Click += new RoutedEventHandler(miDD_Click);
			miDMS.Click += new RoutedEventHandler(miDMS_Click);
			txtX.KeyUp += new KeyEventHandler(txtX_KeyUp);
			txtY.KeyUp += new KeyEventHandler(txtY_KeyUp);
		}

		public void Initialize(Map MyMap, GraphicsLayer MyGraphicsLayer, IParentWindow pParent)
		{

			if (_bInitialized)
				return;

			// Assume map is in meters or feet

			_Map = MyMap;
			_ResultsLayer = MyGraphicsLayer;
			_pParent = pParent;
			_MapUnits = clsHelperCollection.MapUnits;
			_EntryUnits = _MapUnits;
			if (_MapUnits == esriUnits.esriMeters)
				miMapUnits.Header = "Meters";
			else if (_MapUnits == esriUnits.esriFeet)
				miMapUnits.Header = "Feet";
			else
				miMapUnits.Header = "Map Units";
			string sUrl = clsHelperCollection.GeometryHelper.Url;
			_gh = new clsGeometryHelper();
			_gh.Initialize(sUrl);
			_gh.ProjectPointCompleted += new EventHandler<GeometryResultArgs>(_gh_ProjectPointCompleted);
			_gh.Failed += new EventHandler<HelperFailedArgs>(_gh_Failed);
			_CurrentResult = null;
			this.miDMS_Click(null, null);
			_bInitialized = true;

		}

		#region Control Handlers
		private void btnPan_Click(object sender, RoutedEventArgs e)
		{
			_Operation = EntryOperation.Pan;
			if (this.GetEntry())
				this.CompleteOperation();
		}

		private void btnZoom_Click(object sender, RoutedEventArgs e)
		{
			_Operation = EntryOperation.Zoom;
			if (this.GetEntry())
				this.CompleteOperation();
		}

		private void btnFlash_Click(object sender, RoutedEventArgs e)
		{
			_Operation = EntryOperation.Flash;
			if (this.GetEntry())
				this.CompleteOperation();
		}

		private void btnMarker_Click(object sender, RoutedEventArgs e)
		{
			_Operation = EntryOperation.Marker;
			if (this.GetEntry())
				this.CompleteOperation();
		}

		private void btnMarkerXY_Click(object sender, RoutedEventArgs e)
		{
			_Operation = EntryOperation.MarkerXY;
			if (this.GetEntry())
				this.CompleteOperation();
		}

		private void btnBalloon_Click(object sender, RoutedEventArgs e)
		{
			_Operation = EntryOperation.Balloon;
			if (this.GetEntry())
				this.CompleteOperation();
		}

		private void btnClear_Click(object sender, RoutedEventArgs e)
		{
			List<Graphic> DeleteList = new List<Graphic>();
			foreach (Graphic g in _ResultsLayer.Graphics)
			{
				if (!g.Attributes.ContainsKey(sGraphicField))
					continue;
				if (g.Attributes[sGraphicField].ToString() != sGraphicValue)
					continue;
				DeleteList.Add(g);
			}
			foreach (Graphic g in DeleteList)
				_ResultsLayer.Graphics.Remove(g);
		}

		private void btnResults_Click(object sender, RoutedEventArgs e)
		{
			if (cmResults.Items.Count == 0)
				return;
			cmResults.IsOpen = true;
		}

		void btnClearResults_Click(object sender, RoutedEventArgs e)
		{
			_CurrentResult = null;
			cmResults.Items.Clear();
			txtX.Text = "";
			txtY.Text = "";
		}

		private void btnUnits_Click(object sender, RoutedEventArgs e)
		{
			cmUnits.IsOpen = true;
		}

		private void miMapUnits_Click(object sender, RoutedEventArgs e)
		{
			//miMapUnits.IsChecked = true;
			//miDD.IsChecked = false;
			//miDMS.IsChecked = false;
			_EntryUnits = _MapUnits;
			_bDMS = false;
			this.UpdateResults(true);
		}

		private void miDD_Click(object sender, RoutedEventArgs e)
		{
			//miMapUnits.IsChecked = false;
			//miDD.IsChecked = true;
			//miDMS.IsChecked = false;
			_EntryUnits = esriUnits.esriDecimalDegrees;
			_bDMS = false;
			this.UpdateResults(true);
		}

		private void miDMS_Click(object sender, RoutedEventArgs e)
		{
			//miMapUnits.IsChecked = false;
			//miDD.IsChecked = false;
			//miDMS.IsChecked = true;
			_EntryUnits = esriUnits.esriDecimalDegrees;
			_bDMS = true;
			this.UpdateResults(true);
		}

		private void txtX_KeyUp(object sender, KeyEventArgs e)
		{
			if (e.Key != Key.Enter)
				return;
			this.btnPan_Click(null, null);
		}

		void txtY_KeyUp(object sender, KeyEventArgs e)
		{
			if (e.Key != Key.Enter)
				return;
			this.btnPan_Click(null, null);
		}

		void miResult_Click(object sender, RoutedEventArgs e)
		{
			// Update current results
			MenuItem miResult = (MenuItem)sender;
			_CurrentResult = (GoToResult)miResult.Header;
			this.UpdateResults(false);
			clsQueryHelper.Flash(_CurrentResult.ptMap);
		}
		#endregion

		#region Geometry Event Handlers
		private void _gh_ProjectPointCompleted(object sender, GeometryResultArgs e)
		{

			MapPoint pt = (MapPoint)e.Geometry;
			if (_CurrentResult.ptDD == null)
				_CurrentResult.ptDD = pt;
			else
				_CurrentResult.ptMap = pt;
			this.SetHeader(_CurrentResult);

			// Add to results

			MenuItem miResult = new MenuItem();
			miResult.Header = _CurrentResult;
			miResult.Click += new RoutedEventHandler(miResult_Click);
			cmResults.Items.Insert(0, miResult);
			this.UpdateResults(false);
			this.CompleteOperation();

		}

		private void _gh_Failed(object sender, HelperFailedArgs e)
		{
			string sMsg = "Geometry operation failed:\n" + e.Error.Message;
			MessageBox.Show(sMsg, "GoTo", MessageBoxButton.OK);
		}
		#endregion

		#region Private Methods
		private bool GetEntry()
		{

			// Check for valid text box values

			MapPoint ptEntry = null;
			bool bDegrees = (_EntryUnits == esriUnits.esriDecimalDegrees);
			double dX = this.GetEntryAsDouble(txtX);
			if (dX == double.NaN)
			{
				MessageBox.Show("Invalid X or Longitude value");
				return false;
			}
			double dY = this.GetEntryAsDouble(txtY);
			if (dY == double.NaN)
			{
				MessageBox.Show("Invalid Y or Latitude value");
				return false;
			}
			ptEntry = new MapPoint(dX, dY);

			// Check if result is redundant

			if (_CurrentResult != null)
			{
				MapPoint pt;
				if (bDegrees)
					pt = _CurrentResult.ptDD;
				else
					pt = _CurrentResult.ptMap;
				if ((pt.X == ptEntry.X) && (pt.Y == ptEntry.Y))
					return true;
			}

			// Set map point

			_CurrentResult = new GoToResult();
			SpatialReference srDD = new SpatialReference();
			srDD.WKID = 4269; // GCS_North_American_1983
			SpatialReference srTarget;
			if (bDegrees)
			{
				ptEntry.SpatialReference = srDD;
				_CurrentResult.ptDD = ptEntry;
				_CurrentResult.ptMap = null;
				srTarget = _Map.SpatialReference;
			}
			else
			{
				ptEntry.SpatialReference = _Map.SpatialReference;
				_CurrentResult.ptMap = ptEntry;
				_CurrentResult.ptDD = null;
				srTarget = srDD;
			}
			_gh.ProjectPointAsync(ptEntry, srTarget);
			return false;

		}

		private void UpdateResults(bool bUnitsChanged)
		{

			// Update text boxes

			if (!bUnitsChanged)
			{
				string[] Tokens = _CurrentResult.Header.Split(' ');
				txtX.Text = Tokens[0];
				txtY.Text = Tokens[1];
				return;
			}
			txtX.Text = "";
			txtY.Text = "";

			// Update Title

			bool bDegrees = (_EntryUnits == esriUnits.esriDecimalDegrees);
			string sCaption = "Go To XY (";
			if (_bDMS)
				sCaption += "Degrees Minutes Seconds)";
			else if (bDegrees)
				sCaption += "Decimal Degrees)";
			else
				sCaption += miMapUnits.Header.ToString() + ")";
			_pParent.SetTitle(sCaption);

			// Update labels

			if (bDegrees)
			{
				lblX.Content = "Long:";
				lblY.Content = "Lat:";
			}
			else
			{
				lblX.Content = "X:";
				lblY.Content = "Y:";
			}

			// Update result headers

			List<GoToResult> Results = new List<GoToResult>();
			foreach (MenuItem mi in cmResults.Items)
			{
				GoToResult gtResult = (GoToResult)mi.Header;
				this.SetHeader(gtResult);
				Results.Add(gtResult);
			}

			cmResults.Items.Clear();
			foreach (GoToResult gtResult in Results)
			{
				MenuItem miResult = new MenuItem();
				miResult.Header = gtResult;
				miResult.Click += new RoutedEventHandler(miResult_Click);
				cmResults.Items.Add(miResult);
			}
			
		}

		private double GetEntryAsDouble(TextBox txt)
		{
			double d = double.NaN;
			if (string.IsNullOrEmpty(txt.Text))
				return d;
			if (_bDMS)
				d = _gh.FromDMS(txt.Text);
			else
				try
				{
					d = Convert.ToDouble(txt.Text);
				}
				catch
				{
				}
			return d;
		}

		private void SetHeader(GoToResult gtResult)
		{
			bool bDegrees = (_EntryUnits == esriUnits.esriDecimalDegrees);
			if (bDegrees)
			{
				MapPoint ptHeader = gtResult.ptDD;
				if (_bDMS)
					gtResult.Header = _gh.ToDMS(ptHeader.X, true) + " " + _gh.ToDMS(ptHeader.Y, false);
				else
					gtResult.Header = ptHeader.X.ToString("N3") + " " + ptHeader.Y.ToString("N3");
			}
			else
			{
				MapPoint ptHeader = gtResult.ptMap;
				gtResult.Header = ptHeader.X.ToString("N3") + " " + ptHeader.Y.ToString("N3");
			}
		}

		private void CompleteOperation()
		{
			if (_CurrentResult == null)
				return;
			Graphic gr = null;
			switch (_Operation)
			{
				case EntryOperation.Pan:
					clsQueryHelper.Pan(_CurrentResult.ptMap, true);
					break;
				case EntryOperation.Zoom:
					clsQueryHelper.Zoom(_CurrentResult.ptMap, true);					
					break;
				case EntryOperation.Flash:
					clsQueryHelper.Flash(_CurrentResult.ptMap);
					break;
				case EntryOperation.Marker:
					gr = new Graphic();
					gr.Symbol = (Symbol)this.Resources["ResultMarkerSymbol"];
					break;
				case EntryOperation.MarkerXY:
					gr = new Graphic();
					gr.Symbol = (Symbol)this.Resources["ResultXYMarkerSymbol"];
					break;
				case EntryOperation.Balloon:
					gr = new Graphic();
					gr.Symbol = (Symbol)this.Resources["ResultBalloonMarkerSymbol"];
					break;
			}
			if (gr != null)
			{
				gr.Geometry = _CurrentResult.ptMap.Clone();
				gr.Attributes[sGraphicField] = sGraphicValue;
				gr.Attributes["VALUE"] = _CurrentResult.Header;
				_ResultsLayer.Graphics.Add(gr);
			}
		}
		#endregion

	}
}
