﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.FeatureService;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

using FieldApp.Extensibility;
using MapHelpers;

namespace QueryAddin
{

	/// <summary>
	/// Interaction logic for ucIdentify.xaml
	/// </summary>
	public partial class ucIdentify : UserControl
	{

		private const double ExpanderHeight = 25;

		private struct RelTag
		{
			public int iLayerIndex;
			public int iRelatedID;
			public SearchFeature searchSubLayer;
		}

		private double _dExpanderRowHeight = 0;
		private bool _bQueryRelated = false;
		private IParentWindow _pParent;
		private clsCommonHelper _ch;
		private clsGeometryHelper _gh;
		private clsGraphicHelper _grh;
		private Map _Map;
		private esriUnits _MapUnits;
		private esriUnits _DisplayUnits;
		private bool _bDMS;
		private MapPoint _ptLocation;
		private int _iLayerIndex;
		private List<IdentifyTask> _IdentifyTasks;
		private LayerSearchType _SearchType;
		private Geometry _geomSearch;
		private int _iResultCount;
		private bool _bFlashItem = false;
		private bool _bGeometryOnly = false;
		private IdentifyGeometryCallback _callback;
		private ObservableCollection<SearchFeature> _TreeViewData;

		#region Public Methods
		public ucIdentify()
		{
			InitializeComponent();
			expResults.Collapsed += new RoutedEventHandler(expResults_Collapsed);
			expResults.Expanded += new RoutedEventHandler(expResults_Expanded);
			tvFeatures.SelectedItemChanged += new RoutedPropertyChangedEventHandler<object>(tvFeatures_SelectedItemChanged);
		}

		public void Initialize(Map MyMap, bool bQueryRelatedRecords, IParentWindow pParent)
		{

			// Set up events

			_Map = MyMap;
			_pParent = pParent;
			_bQueryRelated = bQueryRelatedRecords;
			string sUrl = clsHelperCollection.GeometryHelper.Url;
			_ch = clsHelperCollection.CommonHelper;
			_gh = new clsGeometryHelper();
			_gh.Initialize(sUrl);
			_gh.GetCenterCompleted += new EventHandler<GeometryResultArgs>(_gh_GetCenterCompleted);
			_gh.ProjectPointCompleted += new EventHandler<GeometryResultArgs>(_gh_ProjectPointCompleted);
			_gh.Failed += new EventHandler<HelperFailedArgs>(_gh_Failed);
			_grh = clsHelperCollection.GraphicHelper;
			_MapUnits = clsHelperCollection.MapUnits;
			_DisplayUnits = _MapUnits;
			QueryBehavior qb = QueryBehavior.Current;
			qb.LayersUpdated += new EventHandler<EventArgs>(qb_LayersUpdated);

			// Set up units menu

			btnUnits.Click += new RoutedEventHandler(btnUnits_Click);
			miMeters.Click += new RoutedEventHandler(miMeters_Click);
			miFeet.Click += new RoutedEventHandler(miFeet_Click);
			miDD.Click += new RoutedEventHandler(miDD_Click);
			miDMS.Click += new RoutedEventHandler(miDMS_Click);

			this.RefreshLayers();

		}

		public void ActivateTool()
		{
			if (_pParent != null)
				_pParent.ShowWindow();
		}

		public void DrawComplete(Geometry g)
		{

			// IdentifyTask doesn't work well with a zero dimension

			double d = _Map.Resolution * 5; // Pixel tolerance
			Envelope env = g.Extent;
			if (env.Width == 0)
			{
				env.XMin -= d;
				env.XMax += d;
			}
			if (env.Height == 0)
			{
				env.YMin -= d;
				env.YMax += d;
			}
			this.DoIdentify(env);

		}

		public void DeactivateTool()
		{
		}

		public FeatureLayerInfo GetSublayerInfo(int iLayerIndex, int iSublayerID)
		{
			if (!clsQueryHelper.Sublayers.ContainsKey(iLayerIndex))
				return null;
			Dictionary<int, FeatureLayerInfo> SublayerInfos = clsQueryHelper.Sublayers[iLayerIndex];
			if (!SublayerInfos.ContainsKey(iSublayerID))
				return null;
			return SublayerInfos[iSublayerID];
		}

		public void IdentifyFeature(SearchFeature searchFeat)
		{

			// Clear tree view and populate with feature
			// Note: LayerIndex and LayerName must be populated

			searchFeat.IsExpanded = false;
			searchFeat.IsSelected = true;
			tvFeatures.ItemsSource = null;
			dgFields.ItemsSource = null;
			_iLayerIndex = searchFeat.LayerIndex;
			SearchFeature searchSubLayer = new SearchFeature();
			searchSubLayer.Value = searchFeat.LayerName;
			searchSubLayer.IsExpanded = true;
			searchSubLayer.IsSelected = false;
			searchSubLayer.AddRelatedRecord(searchFeat);
			SearchFeature searchLayer = new SearchFeature();
			searchLayer.Value = clsQueryHelper.DynamicLayers[searchFeat.LayerIndex].ID;
			searchLayer.IsExpanded = true;
			searchLayer.IsSelected = false;
			searchLayer.AddRelatedRecord(searchSubLayer);
			_TreeViewData = new ObservableCollection<SearchFeature>();
			_TreeViewData.Add(searchLayer);
			_bFlashItem = true;
			tvFeatures.ItemsSource = _TreeViewData;
			_pParent.ShowWindow();

		}

		public void IdentifyGeometry(MapPoint pt, IdentifyGeometryCallback callback)
		{
			_bGeometryOnly = true;
			_callback = callback;
			this.DoIdentify(pt);
		}
		#endregion

		#region XAML Declared Event Handlers
		private void miFlash_Click(object sender, RoutedEventArgs e)
		{
			MenuItem mi = (MenuItem)sender;
			SearchFeature searchFeat = (SearchFeature)mi.DataContext;
			Geometry g = searchFeat.Feature.Geometry;
			clsQueryHelper.Flash(g, true);
		}

		private void miZoomTo_Click(object sender, RoutedEventArgs e)
		{
			MenuItem mi = (MenuItem)sender;
			SearchFeature searchFeat = (SearchFeature)mi.DataContext;
			Geometry g = searchFeat.Feature.Geometry;
			clsQueryHelper.Zoom(g, true);
		}

		private void miPanTo_Click(object sender, RoutedEventArgs e)
		{
			MenuItem mi = (MenuItem)sender;
			SearchFeature searchFeat = (SearchFeature)mi.DataContext;
			Geometry g = searchFeat.Feature.Geometry;
			clsQueryHelper.Pan(g, true);
		}
		#endregion

		#region Event Handlers
		private void expResults_Collapsed(object sender, RoutedEventArgs e)
		{
			_dExpanderRowHeight = grdIdentify.RowDefinitions[1].ActualHeight;
			grdIdentify.RowDefinitions[1].Height = new GridLength(ExpanderHeight);
			gsResults.IsEnabled = false;
		}

		private void expResults_Expanded(object sender, RoutedEventArgs e)
		{
			if (_dExpanderRowHeight == 0)
				return;
			grdIdentify.RowDefinitions[1].Height = new GridLength(_dExpanderRowHeight);
			gsResults.IsEnabled = true;
		}

		private void tvFeatures_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
		{

			// Get feature

			object oVal = e.NewValue;
			if (oVal == null)
				return;
			if (!(oVal is SearchFeature))
				return;
			SearchFeature idFeat = (SearchFeature)oVal;
			Graphic gFeat = idFeat.Feature;

			// Display attributes

			dgFields.ItemsSource = gFeat.Attributes;

			// Get location and optionally flash feature

			_ptLocation = null;
			if (gFeat.Geometry != null)
			{
				_gh.GetCenterAsync(gFeat.Geometry, gFeat.Geometry);
				return;
			}
			_bFlashItem = true;
			this.SetLocation();

		}

		private void IdTask_ExecuteCompleted(object sender, IdentifyEventArgs e)
		{

			// Check for no results

			bool bNoResults = false;
			if (e.IdentifyResults == null)
				bNoResults = true;
			else if (e.IdentifyResults.Count == 0)
				bNoResults = true;
			if (bNoResults)
			{

				// See if next dynamic layer needs to be queried

				if (_SearchType != LayerSearchType.Single)
				{
					if (_iLayerIndex < clsQueryHelper.DynamicLayers.Count - 1)
					{
						_iLayerIndex += 1;
						IdentifyParameters idParam = GetIdentifyParameters();
						IdentifyTask idTask = _IdentifyTasks[_iLayerIndex];
						idTask.ExecuteAsync(idParam);
						return;
					}
				}

				// Nothing found

				this.Status("No features found.");
				if (_bGeometryOnly)
				{
					_bGeometryOnly = false;
					_callback(null);
				}
				return;

			}

			// Build results list

			this.Status("Building results...");
			Dictionary<string, List<SearchFeature>> ResultsList = new Dictionary<string, List<SearchFeature>>();
			Dictionary<int, string> SublayerNameList = new Dictionary<int, string>();
			foreach (IdentifyResult idResult in e.IdentifyResults)
			{
				if (_bGeometryOnly)
				{
					this.Status("Identified 1 feature");
					_bGeometryOnly = false;
					_callback(idResult.Feature.Geometry);
					return;
				}
				string sSublayerName = idResult.LayerName;
				SearchFeature searchFeat = new SearchFeature();
				searchFeat.IsExpanded = false;
				searchFeat.IsSelected = false;
				searchFeat.LayerIndex = _iLayerIndex; // Not used here
				searchFeat.LayerName = sSublayerName;
				searchFeat.FieldName = idResult.DisplayFieldName; // Not used here
				if (idResult.Value == null)
					searchFeat.Value = "";
				else
					searchFeat.Value = idResult.Value.ToString();
				Graphic gFeat = idResult.Feature;
				searchFeat.Feature = gFeat;
				if (gFeat.Attributes.ContainsKey("OBJECTID"))
					searchFeat.ObjectID = Convert.ToInt32(gFeat.Attributes["OBJECTID"]);
				List<SearchFeature> FeatList;
				if (!ResultsList.ContainsKey(sSublayerName))
				{
					FeatList = new List<SearchFeature>();
					ResultsList[sSublayerName] = FeatList;
					SublayerNameList[idResult.LayerId] = sSublayerName;
				}
				else
					FeatList = ResultsList[sSublayerName];
				FeatList.Add(searchFeat);
				_grh.FlashGeometry(_Map, idResult.Feature.Geometry, null);
			}
			List<int> IDSortList = new List<int>();
			foreach (int iKey in SublayerNameList.Keys)
				IDSortList.Add(iKey);
			IDSortList.Sort();

			// Populate tree view data

			SearchFeature searchLayer = new SearchFeature();
			searchLayer.Value = clsQueryHelper.DynamicLayers[_iLayerIndex].ID;
			searchLayer.IsExpanded = true;
			searchLayer.IsSelected = false;
			foreach (int iKey in IDSortList)
			{
				string sSublayerName = SublayerNameList[iKey];
				SearchFeature searchSubLayer = new SearchFeature();
				searchSubLayer.Value = sSublayerName;
				searchSubLayer.IsExpanded = true;
				searchSubLayer.IsSelected = false;
				List<SearchFeature> FeatList = ResultsList[sSublayerName];
				foreach (SearchFeature searchFeat in FeatList)
				{
					searchSubLayer.AddRelatedRecord(searchFeat);
					_iResultCount++;
				}
				searchLayer.AddRelatedRecord(searchSubLayer);
				if (_bQueryRelated)
					this.GetRelatedRecords(iKey, searchSubLayer);
			}
			_TreeViewData.Add(searchLayer);

			// See if next dynamic layer needs to be queried

			if ((_SearchType == LayerSearchType.Visible) || (_SearchType == LayerSearchType.All))
			{
				if (_iLayerIndex < clsQueryHelper.DynamicLayers.Count - 1)
				{
					_iLayerIndex += 1;
					IdentifyParameters idParam = GetIdentifyParameters();
					IdentifyTask idTask = _IdentifyTasks[_iLayerIndex];
					idTask.ExecuteAsync(idParam);
					return;
				}
			}

			// Done

			string sMsg = "Identified " + _iResultCount + " feature";
			if (_iResultCount > 1)
				sMsg += "s";
			this.Status(sMsg);

			// Select first feature and populate tree view

			searchLayer = _TreeViewData[0];
			SearchFeature subLayer = searchLayer.RelatedRecords[0];
			SearchFeature sfFeat = subLayer.RelatedRecords[0];
			sfFeat.IsSelected = true;
			_bFlashItem = false;
			tvFeatures.ItemsSource = _TreeViewData;

		}

		private void IdTask_Failed(object sender, TaskFailedEventArgs e)
		{
			this.Error("Identify failed:\n" + e.Error.Message);
		}

		private void queryTask_ExecuteRelationshipQueryCompleted(object sender, RelationshipEventArgs e)
		{

			RelTag relTag = (RelTag)e.UserState;
			int iLayerIndex = relTag.iLayerIndex;
			SearchFeature searchSubLayer = relTag.searchSubLayer;
			int iRelID = relTag.iRelatedID;
			FeatureLayerInfo Info = this.GetSublayerInfo(iLayerIndex, iRelID);
			string sRelName = Info.Name;
			string sDisplayField = Info.DisplayField;

			// Add related items

			IDictionary<int, IEnumerable<Graphic>> relRecords = e.Result.RelatedRecordsGroup;
			foreach (SearchFeature sfItem in searchSubLayer.RelatedRecords)
			{

				int iOID = sfItem.ObjectID;
				if (!relRecords.ContainsKey(iOID))
					continue;

				// Create related layer item

				SearchFeature searchRelSubLayer = new SearchFeature();
				searchRelSubLayer.Value = sRelName;
				searchRelSubLayer.IsExpanded = false;
				searchRelSubLayer.IsSelected = false;
				sfItem.AddRelatedRecord(searchRelSubLayer);

				// Add related records

				IEnumerable<Graphic> recList = relRecords[iOID];
				foreach (Graphic gFeat in recList)
				{
					SearchFeature sfRel = new SearchFeature();
					sfRel.IsExpanded = false;
					sfRel.IsSelected = false;
					sfRel.Feature = gFeat;
					if (gFeat.Attributes.ContainsKey("OBJECTID"))
						sfRel.ObjectID = Convert.ToInt32(gFeat.Attributes["OBJECTID"]);
					sfRel.LayerName = sRelName;
					sfRel.FieldName = sDisplayField;
					if (gFeat.Attributes.ContainsKey(sDisplayField))
					{
						if (gFeat.Attributes[sDisplayField] != null)
							sfRel.Value = gFeat.Attributes[sDisplayField].ToString();
						else
							sfRel.Value = sfRel.ObjectID.ToString();
					}
					else
						sfRel.Value = sfRel.ObjectID.ToString();

					searchRelSubLayer.AddRelatedRecord(sfRel);
				}

			}

		}

		private void queryTask_Failed(object sender, TaskFailedEventArgs e)
		{
			this.Error("Query related records failed:\n" + e.Error.Message);
		}

		private void _gh_GetCenterCompleted(object sender, GeometryResultArgs e)
		{
			_ptLocation = (MapPoint)e.Geometry;
			Geometry gFeat = (Geometry)e.UserState;
			if (_bFlashItem)
				_grh.FlashGeometry(_Map, gFeat, _ptLocation);
			_bFlashItem = true;
			this.SetLocation();
		}

		private void _gh_ProjectPointCompleted(object sender, GeometryResultArgs e)
		{
			MapPoint pt = (MapPoint)e.Geometry;
			double dX = pt.X;
			double dY = pt.Y;
			string sLoc;
			if (_bDMS)
			{
				sLoc = _gh.ToDMS(dX, true) + " " + _gh.ToDMS(dY, false);
				tbLocation.Text = sLoc;
				return;
			}
			this.DisplayXY(dX, dY);
		}

		private void _gh_Failed(object sender, HelperFailedArgs e)
		{
			this.Error("Geometry operation failed:\n" + e.Error.Message);
		}

		private void btnUnits_Click(object sender, RoutedEventArgs e)
		{
			// Comment out the following line for Silverlight
			btnUnits.ContextMenu.IsOpen = true;
		}

		private void miMeters_Click(object sender, RoutedEventArgs e)
		{
			_DisplayUnits = esriUnits.esriMeters;
			this.SetLocation();
		}

		private void miFeet_Click(object sender, RoutedEventArgs e)
		{
			_DisplayUnits = esriUnits.esriFeet;
			this.SetLocation();
		}

		private void miDD_Click(object sender, RoutedEventArgs e)
		{
			_DisplayUnits = esriUnits.esriDecimalDegrees;
			_bDMS = false;
			this.SetLocation();
		}

		private void miDMS_Click(object sender, RoutedEventArgs e)
		{
			_DisplayUnits = esriUnits.esriDecimalDegrees;
			_bDMS = true;
			this.SetLocation();
		}

		void qb_LayersUpdated(object sender, EventArgs e)
		{
			this.RefreshLayers();
		}
		#endregion

		#region Private Methods
		private void Status(string sMsg)
		{
			lblStatus.Content = sMsg;
			//lblStatus.Refresh(); // Silverlight only
		}

		private void Error(string sErr)
		{
			lblStatus.Content = "ERROR";
			MessageBox.Show(sErr, "ERROR", MessageBoxButton.OK);
		}

		private void RefreshLayers()
		{

			tvFeatures.ItemsSource = null;
			dgFields.ItemsSource = null;

			// Set up tasks

			_IdentifyTasks = new List<IdentifyTask>();
			_iLayerIndex = 0;
			foreach (ArcGISDynamicMapServiceLayer dynLayer in clsQueryHelper.DynamicLayers)
			{
				string sURL = dynLayer.Url;
				IdentifyTask IdTask = new IdentifyTask(sURL);
				IdTask.ExecuteCompleted += new EventHandler<IdentifyEventArgs>(IdTask_ExecuteCompleted);
				IdTask.Failed += new EventHandler<TaskFailedEventArgs>(IdTask_Failed);
				_IdentifyTasks.Add(IdTask);
			}

			// Set up layer options

			cbOptions.Items.Clear();
			LayerSearchType[] Types = { LayerSearchType.Topmost, LayerSearchType.Visible, LayerSearchType.All };
			foreach (LayerSearchType opt in Types)
			{
				LayerSearchItem lsItem = new LayerSearchItem();
				lsItem.SearchType = opt;
				cbOptions.Items.Add(lsItem);
			}
			foreach (QueryLayer ql in clsQueryHelper.QueryableLayers)
			{
				LayerSearchItem lsItem = new LayerSearchItem();
				lsItem.SearchType = LayerSearchType.Single;
				lsItem.LayerIndex = ql.LayerIndex;
				lsItem.SubLayerID = ql.SublayerID;
				lsItem.SubLayerName = ql.SublayerName;
				cbOptions.Items.Add(lsItem);
			}
			cbOptions.SelectedIndex = 0;
			this.Status("Identified 0 features");

		}

		private void DoIdentify(Geometry g)
		{

			object oVal = cbOptions.SelectedItem;
			if (oVal == null)
				return;
			if (!(oVal is LayerSearchItem))
				return; // This should never happen

			// Set up query

			tvFeatures.ItemsSource = null;
			_TreeViewData = new ObservableCollection<SearchFeature>();
			expResults.IsExpanded = true;
			dgFields.ItemsSource = null;
			if (!_bGeometryOnly)
			{
				_pParent.ShowWindow();
				this.Status("Querying...");
			}
			_iResultCount = 0;
			_iLayerIndex = 0;
			_geomSearch = g;
			IdentifyParameters idParam = this.GetIdentifyParameters();
			IdentifyTask idTask = _IdentifyTasks[_iLayerIndex];
			idTask.ExecuteAsync(idParam);

		}

		private IdentifyParameters GetIdentifyParameters()
		{

			object oVal = cbOptions.SelectedItem;
			LayerSearchItem lsItem = (LayerSearchItem)oVal;
			if (_bGeometryOnly)
				_SearchType = LayerSearchType.Topmost;
			else
				_SearchType = lsItem.SearchType;
			IdentifyParameters idParam = new IdentifyParameters();
			idParam.Geometry = _geomSearch;
			idParam.ReturnGeometry = true;
			idParam.MapExtent = _Map.Extent;
			idParam.Height = (int)_Map.ActualHeight;
			idParam.Width = (int)_Map.ActualWidth;
			idParam.SpatialReference = _Map.SpatialReference;
			List<int> IDLayers = idParam.LayerIds;
			IDLayers.Clear();

			bool bUseVis = false;
			switch (_SearchType)
			{
				case LayerSearchType.Topmost:
					idParam.LayerOption = LayerOption.top;
					bUseVis = true;
					break;
				case LayerSearchType.Visible:
					idParam.LayerOption = LayerOption.visible;
					bUseVis = true;
					break;
				case LayerSearchType.All:
					idParam.LayerOption = LayerOption.all;
					break;
				default:
					idParam.LayerOption = LayerOption.all;
					_iLayerIndex = lsItem.LayerIndex;
					IDLayers.Add(lsItem.SubLayerID);
					return idParam;
			}

			// Restrict layers to those that are queryable

			ArcGISDynamicMapServiceLayer ds = clsQueryHelper.DynamicLayers[_iLayerIndex];
			double dScale = _Map.Scale;
			foreach (QueryLayer ql in clsQueryHelper.QueryableLayers)
			{
				if (ql.LayerIndex != _iLayerIndex)
					continue;
				int iSubLayer = ql.SublayerID;
				if (bUseVis)
				{
					if (!_ch.IsSublayerVisible(ds, iSubLayer, dScale))
						continue;
				}
				IDLayers.Add(iSubLayer);
			}
			return idParam;

		}

		private void GetRelatedRecords(int iSublayerID, SearchFeature searchSublayer)
		{

			// Get sublayer details

			FeatureLayerInfo Info = this.GetSublayerInfo(_iLayerIndex, iSublayerID);
			if (Info == null)
				return;  // This should never happen
			if (Info.Relationships == null)
				return;

			// Get feature ids

			List<int> IDList = new List<int>();
			foreach (SearchFeature sfItem in searchSublayer.RelatedRecords)
			{
				int iOID = sfItem.ObjectID;
				if (iOID <= 0)
					continue;
				IDList.Add(iOID);
			}

			// Get related records by relationship ID

			Dictionary<int, List<TreeViewItem>> Results = new Dictionary<int, List<TreeViewItem>>();
			foreach (Relationship Rel in Info.Relationships)
			{

				int iRelID = Rel.Id;
				int iRelLayerID = Rel.RelatedTableId;

				// Set up query task

				ArcGISDynamicMapServiceLayer dynLayer = clsQueryHelper.DynamicLayers[_iLayerIndex];
				string sURL = dynLayer.Url;
				sURL += "/" + iSublayerID.ToString();
				QueryTask queryTask = new QueryTask(sURL);
				queryTask.ExecuteRelationshipQueryCompleted += new EventHandler<RelationshipEventArgs>(queryTask_ExecuteRelationshipQueryCompleted);
				queryTask.Failed += new EventHandler<TaskFailedEventArgs>(queryTask_Failed);

				// Set up query parameters

				RelationshipParameter relParams = new RelationshipParameter();
				relParams.RelationshipId = iRelID;
				relParams.ObjectIds = IDList;
				relParams.ReturnGeometry = true;
				relParams.OutSpatialReference = _Map.SpatialReference;
				relParams.OutFields = new string[] { "*" };

				// Execute query

				RelTag userToken = new RelTag();
				userToken.iLayerIndex = _iLayerIndex;
				userToken.iRelatedID = iRelLayerID;
				userToken.searchSubLayer = searchSublayer;
				queryTask.ExecuteRelationshipQueryAsync(relParams, userToken);

			}

		}

		private void SetLocation()
		{
			if (_ptLocation == null)
			{
				tbLocation.Text = "";
				return;
			}
			double dX = 0;
			double dY = 0;
			if (_DisplayUnits == esriUnits.esriDecimalDegrees)
			{
				SpatialReference srMap = _Map.SpatialReference;
				SpatialReference sr = new SpatialReference();
				if ((srMap.WKID == 102100) || (srMap.WKID == 3857))
					sr.WKID = 4326; // GCS_WGS_1984
				else
					sr.WKID = 4269; // GCS_North_American_1983
				_gh.ProjectPointAsync(_ptLocation, sr);
				return;
			}
			else
			{
				dX = _gh.ConvertLinearUnit(_ptLocation.X, _MapUnits, _DisplayUnits);
				dY = _gh.ConvertLinearUnit(_ptLocation.Y, _MapUnits, _DisplayUnits);
			}
			this.DisplayXY(dX, dY);
		}

		private void DisplayXY(double dX, double dY)
		{
			string sLoc = "";
			sLoc = dX.ToString("N3") + " " + dY.ToString("N3");
			switch (_DisplayUnits)
			{
				case esriUnits.esriMeters:
					sLoc += " Meters";
					break;
				case esriUnits.esriFeet:
					sLoc += " Feet";
					break;
				case esriUnits.esriDecimalDegrees:
					sLoc += " Decimal Degrees";
					break;
			}
			tbLocation.Text = sLoc;
		}
		#endregion


	}
}
