﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Local;

using FieldApp.Extensibility;
using MapHelpers;

namespace QueryAddin
{
	public class QueryBehavior : IBehavior
	{

		public static QueryBehavior Current { get; private set; }
		public static GraphicsLayer ResultsLayer { get; private set; }
		public static ucIdentify IdentifyEngine { get; private set; }
		public bool IsInitialized { get; private set; }

		private clsQueryHelper _qHelper;

		public event EventHandler<EventArgs> Initialized;
		public event EventHandler<EventArgs> LayersUpdated;
		
		#region Initialization
		public void OnAttached()
		{

			Current = this;
			IsInitialized = false;

			// Create identify control

			IdentifyEngine = new ucIdentify();

			// Set up query helper

			_qHelper = new clsQueryHelper();
			_qHelper.QueryHelperInitialized += new System.EventHandler<System.EventArgs>(QueryHelperInitialized);
			_qHelper.QueryHelperUpdated += new EventHandler<EventArgs>(QueryHelperUpdated);

			// Add layer initialization listener

			Map MyMap = MapApplication.Current.Map;
			LayerCollection MyLayers = MyMap.Layers;
			MyLayers.LayersInitialized += new LayerCollection.LayersInitializedHandler(MyLayers_LayersInitialized);

		}
		#endregion

		private void MyLayers_LayersInitialized(object sender, EventArgs args)
		{

			if (IsInitialized)
				return;

			Map MyMap = MapApplication.Current.Map;
			clsHelperCollection.InitializeHelpers();

			// Get map units

			clsCommonHelper ch = clsHelperCollection.CommonHelper;
			clsHelperCollection.MapUnits = ch.GetMapUnits(MyMap);

			// Initialize geometry helper and query helper

			LocalGeometryService.GetServiceAsync(localService =>
			{
				clsGeometryHelper geomHelper = clsHelperCollection.GeometryHelper;
				if (!geomHelper.Initialized)
					geomHelper.Initialize(localService.UrlGeometryService);
				_qHelper.Init(MyMap);
			});

		}

		private void MyLayers_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
		{
			_qHelper.UpdateLayers();
		}

		private void QueryHelperInitialized(object sender, System.EventArgs e)
		{

			// Add results graphics layer (for flash, GoToXY, [highlight?])

			Map MyMap = MapApplication.Current.Map;
			LayerCollection MyLayers = MyMap.Layers;
			ResultsLayer = new GraphicsLayer();
			ResultsLayer.ID = "QueryAddinResultsLayer";
			SimpleRenderer sr = new SimpleRenderer();
			sr.Symbol = new ESRI.ArcGIS.Client.Symbols.SimpleMarkerSymbol();
			ResultsLayer.RendererTakesPrecedence = false;
			ResultsLayer.Visible = true;
			MyLayers.Add(ResultsLayer);
			MyLayers.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(MyLayers_CollectionChanged);
			IsInitialized = true;
			if (this.Initialized != null)
				this.Initialized(this, new EventArgs());

		}

		private void QueryHelperUpdated(object sender, EventArgs e)
		{
			if (this.LayersUpdated != null)
				this.LayersUpdated(this, new EventArgs());
		}
	
	}
}
