﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Input;
using System.Windows.Media;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Symbols;

using FieldApp.Extensibility;
using MapHelpers;

namespace QueryAddin
{

	public class IdentifyTool : QueryTool
	{

		private const string sWindowIcon = "QueryAddin.Images.Info.ico";

		private ucIdentify _uc;
		private Draw _Draw;

		public IdentifyTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sButtonContent = null;
			_sCursorPath = "QueryAddin.Images.Identify.cur";
			_sIconPath = "QueryAddin.Images.Identify.png";
			_sToolTip = "Identify";
		}

		protected override void QInitialized()
		{
			Map MyMap = MapApplication.Current.Map;
			ParentWindow pw = new ParentWindow();
			_uc = QueryBehavior.IdentifyEngine;
			_uc.Initialize(MyMap, true, pw); // Set true to query related records
			pw.SetContent(_uc, true);
			pw.SizeToContent = System.Windows.SizeToContent.Manual;
			pw.Height = 350;
			pw.Width = 275;
			pw.SetTitle("Identify");
			pw.SetIcon(AddinHelper.GetImageSource(_MyAssembly, _sIconPath));
			_Draw = new Draw(MyMap);
			_Draw.DrawMode = DrawMode.Point;
			_Draw.DrawComplete += new EventHandler<DrawEventArgs>(_Draw_DrawComplete);
		}

		protected override void QActivateTool()
		{
			_Draw.IsEnabled = true;
			_uc.ActivateTool();
		}

		protected override void QDeactivateTool()
		{
			_Draw.IsEnabled = false;
			_uc.DeactivateTool();
		}

		void _Draw_DrawComplete(object sender, DrawEventArgs e)
		{
			_uc.DrawComplete(e.Geometry);	
		}

	}

	public class FindButton : QueryButton
	{

		private const string sWindowIcon = "QueryAddin.Images.Find.ico";

		private static IParentWindow _pw = null;

		public static void Launch()
		{

			// Accessible from FindMenuItem

			if (_pw == null)
			{
				ucFind uc = new ucFind();
				uc.Initialize(MapApplication.Current.Map, QueryBehavior.IdentifyEngine);
				ParentWindow pw = new ParentWindow();
				pw.SetContent(uc, true);
				pw.SizeToContent = System.Windows.SizeToContent.WidthAndHeight;
				pw.SetTitle("Find");
				pw.SetIcon(AddinHelper.GetImageSource(Assembly.GetExecutingAssembly(), sWindowIcon));
				_pw = pw;
			}
			_pw.ShowWindow();

		}

		public FindButton()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sButtonContent = null;
			_sIconPath = "QueryAddin.Images.Find.png";
			_sToolTip = "Find";
		}

		protected override void QExecute()
		{
			Launch();
		}

	}

	public class GoToXYButton : QueryButton
	{

		private static IParentWindow _pw = null;

		public static void Launch()
		{

			// Accessible from GoToXYMenuItem

			if (_pw == null)
			{
				ucGoToXY uc = new ucGoToXY();
				ParentWindow pw = new ParentWindow();
				uc.Initialize(MapApplication.Current.Map, QueryBehavior.ResultsLayer, pw);
				pw.SetContent(uc, false);
				pw.SetTitle("Go To XY (Degrees Minutes Seconds)");
				pw.SetIcon(null);
				_pw = pw;
			}
			_pw.ShowWindow();

		}

		public GoToXYButton()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sButtonContent = null;
			_sIconPath = "QueryAddin.Images.FindXY.png";
			_sToolTip = "Go To XY";
		}

		protected override void QExecute()
		{
			Launch();
		}

	}

	public class MeasureTool : QueryTool
	{

		private ucMeasure _uc;

		public MeasureTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sButtonContent = null;
			_sCursorPath = "QueryAddin.Images.Measure.cur";
			_sIconPath = "QueryAddin.Images.Measure.png";
			_sToolTip = "Measure";
		}

		protected override void QInitialized()
		{
			Map MyMap = MapApplication.Current.Map;
			_uc = new ucMeasure();
			ParentWindow pw = new ParentWindow();
			pw.SetContent(_uc, false);
			pw.SetTitle("Measure");
			pw.SetIcon(null);
			_uc.Initialize(MyMap, pw, QueryBehavior.IdentifyEngine);
		}

		protected override void QActivateTool()
		{
			_uc.ActivateTool();
		}

		protected override void QDeactivateTool()
		{
			_uc.DeactivateTool();
		}

	}

	public class HighlightTool : QuerySplitTool
	{

		public static clsHighlighter Highlighter;

		private const string sYellowIcon = "QueryAddin.Images.HighlightYellow.png";
		private const string sBlueIcon = "QueryAddin.Images.HighlightBlue.png";
		private const string sGreenIcon = "QueryAddin.Images.HighlightGreen.png";
		private const string sYellowCursor = "QueryAddin.Images.HighlighterYellow.cur";
		private const string sBlueCursor = "QueryAddin.Images.HighlighterBlue.cur";
		private const string sGreenCursor = "QueryAddin.Images.HighlighterGreen.cur";

		private HighlighterToolType _htType;

		public HighlightTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sButtonContent = null;
			_sCursorPath = sYellowCursor;
			_sIconPath = sYellowIcon;
			_sDropIconPath = "QueryAddin.Images.Down.png";
			_sToolTip = "Yellow Highlighter";
			_htType = HighlighterToolType.HighlightYellow;
		}

		protected override void QInitialized()
		{
			Highlighter = new clsHighlighter();
			Highlighter.Initialize(MapApplication.Current.Map, QueryBehavior.ResultsLayer);
		}

		protected override void  QActivateTool()
		{
			Highlighter.SetTool(_htType);
		}

		protected override void  QDeactivateTool()
		{
			Highlighter.SetTool(HighlighterToolType.None);
		}

		protected override void QOnItemSelected(IMapMenuItem item)
		{
			if (item is HighlightYellowMenuItem)
			{
				_htType = HighlighterToolType.HighlightYellow;
				_sCursorPath = sYellowCursor;
				_sIconPath = sYellowIcon;
				_sToolTip = "Yellow Highlighter";
			}
			else if (item is HighlightBlueMenuItem)
			{
				_htType = HighlighterToolType.HighlightBlue;
				_sCursorPath = sBlueCursor;
				_sIconPath = sBlueIcon;
				_sToolTip = "Blue Highlighter";
			}
			else
			{
				_htType = HighlighterToolType.HighlightGreen;
				_sCursorPath = sGreenCursor;
				_sIconPath = sGreenIcon;
				_sToolTip = "Green Highlighter";
			}
			Highlighter.SetTool(_htType);
		}

	}

	public class EraserTool : QuerySplitTool
	{

		private const string sEntireIcon = "QueryAddin.Images.EraseEntire.png";
		private const string sSmallIcon = "QueryAddin.Images.EraseSmall.png";
		private const string sMediumIcon = "QueryAddin.Images.EraseMedium.png";
		private const string sLargeIcon = "QueryAddin.Images.EraseLarge.png";
		private const string sEntireCursor = "QueryAddin.Images.Eraser.cur";
		private const string sSmallCursor = "QueryAddin.Images.EraserSmall.cur";
		private const string sMediumCursor = "QueryAddin.Images.EraserMedium.cur";
		private const string sLargeCursor = "QueryAddin.Images.EraserLarge.cur";

		private HighlighterToolType _htType;

		public EraserTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sButtonContent = null;
			_sCursorPath = sEntireCursor;
			_sIconPath = sEntireIcon;
			_sDropIconPath = "QueryAddin.Images.Down.png";
			_sToolTip = "Erase Entire Ink Stroke";
			_htType = HighlighterToolType.EraserEntire;
		}

		protected override void QInitialized()
		{
		}

		protected override void QActivateTool()
		{
			HighlightTool.Highlighter.SetTool(_htType);
		}

		protected override void QDeactivateTool()
		{
			HighlightTool.Highlighter.SetTool(HighlighterToolType.None);
		}

		protected override void QOnItemSelected(IMapMenuItem item)
		{
			if (item is EraseEntireMenuItem)
			{
				_htType = HighlighterToolType.EraserEntire;
				_sCursorPath = sEntireCursor;
				_sIconPath = sEntireIcon;
				_sToolTip = "Erase Entire Ink Stroke";
			}
			else if (item is EraseSmallMenuItem)
			{
				_htType = HighlighterToolType.EraserSmall;
				_sCursorPath = sSmallCursor;
				_sIconPath = sSmallIcon;
				_sToolTip = "Erase Using Small Eraser";
			}
			else if (item is EraseMediumMenuItem)
			{
				_htType = HighlighterToolType.EraserMedium;
				_sCursorPath = sMediumCursor;
				_sIconPath = sMediumIcon;
				_sToolTip = "Erase Using Medium Eraser";
			}
			else
			{
				_htType = HighlighterToolType.EraserLarge;
				_sCursorPath = sLargeCursor;
				_sIconPath = sLargeIcon;
				_sToolTip = "Erase Using Medium Eraser";
			}
			HighlightTool.Highlighter.SetTool(_htType);
			
		}

	}


}
