﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;

namespace FieldApp.Extensibility
{

	static class ResourceStyleKeys
	{
		public const string MenuItemStyle = "LayoutMenuItemStyle";
		public const string ButtonStyle = "LayoutButtonStyle";
		public const string ButtonImageStyle = "LayoutButtonImageStyle";
		public const string ToolStyle = "LayoutToolStyle";
		public const string DropButtonStyle = "LayoutDropButtonStyle";
		public const string DropButtonImageStyle = "LayoutButtonImageStyle";
		public const string ComboBoxStyle = "LayoutComboBoxStyle";
	}

	class MapMenuItem : MenuItem
	{

		private IMapMenuItem _pMenuItem;

		public IMapMenuItem MenuItem { get { return _pMenuItem; } }

		public MapMenuItem(IMapMenuItem pMenuItem)
		{

			_pMenuItem = pMenuItem;
			_pMenuItem.IsEnabledChanged += new EventHandler<EventArgs>(_pMenuItem_IsEnabledChanged);
			ResourceDictionary rd = MapApplication.Current.Resources;
			if (rd != null)
			{
				string sKey = ResourceStyleKeys.MenuItemStyle;
				if (rd.Contains(sKey))
					base.Style = (Style)rd[sKey];
			}
			base.Icon = _pMenuItem.Icon;
			base.IsCheckable = _pMenuItem.IsCheckable;
			if (_pMenuItem.IsCheckable)
				base.IsChecked = _pMenuItem.IsChecked;
			base.IsEnabled = _pMenuItem.IsEnabled;
			base.Header = _pMenuItem.Label;

		}

		private void _pMenuItem_IsEnabledChanged(object sender, EventArgs e)
		{
			base.IsEnabled = _pMenuItem.IsEnabled;
		}

		protected override void OnClick()
		{
			base.OnClick();
			_pMenuItem.OnClick();
			if (_pMenuItem.IsCheckable)
				base.IsChecked = _pMenuItem.IsChecked;
		}

	}

	class MapButton : Button
	{

		private IMapButton _pButton;

		public MapButton(IMapButton pButton)
		{

			_pButton = pButton;
			_pButton.IsEnabledChanged += new EventHandler<EventArgs>(_pButton_IsEnabledChanged);
			if (_pButton is IMapSplitButton)
			{
				IMapSplitButton pSplitButton = (IMapSplitButton)_pButton;
				pSplitButton.SelectionChanged += new EventHandler<EventArgs>(pSplitButton_SelectionChanged);
			}
			ResourceDictionary rd = MapApplication.Current.Resources;
			if (rd != null)
			{
				string sKey = ResourceStyleKeys.ButtonStyle;
				if (rd.Contains(sKey))
					base.Style = (Style)rd[sKey];
			}
			base.IsEnabled = _pButton.IsEnabled;
			this.SetAppearance();

		}

		protected override void OnClick()
		{
			base.OnClick();
			_pButton.OnClick();
		}

		private void _pButton_IsEnabledChanged(object sender, EventArgs e)
		{
			base.IsEnabled = _pButton.IsEnabled;
		}

		private void pSplitButton_SelectionChanged(object sender, EventArgs e)
		{
			this.SetAppearance();
		}

		private void SetAppearance()
		{
			ResourceDictionary rd = MapApplication.Current.Resources;
			object obj = _pButton.ButtonContent;
			if ((rd != null) && (obj is Image))
			{
				Image img = (Image)obj;
				string sKey = ResourceStyleKeys.ButtonImageStyle;
				if (rd.Contains(sKey))
					img.Style = (Style)rd[sKey];
			}
			base.Content = obj;
			base.ToolTip = _pButton.ToolTip;
		}

	}

	class MapTool : RadioButton
	{

		private IMapTool _pTool;

		public event EventHandler<EventArgs> Activated;

		public MapTool(IMapTool pTool)
		{

			_pTool = pTool;
			_pTool.IsEnabledChanged += new EventHandler<EventArgs>(_pTool_IsEnabledChanged);
			if (_pTool is IMapSplitButton)
			{
				IMapSplitButton pSplitButton = (IMapSplitButton)_pTool;
				pSplitButton.SelectionChanged += new EventHandler<EventArgs>(pSplitButton_SelectionChanged);
			}
			ResourceDictionary rd = MapApplication.Current.Resources;
			if (rd != null)
			{
				string sKey = ResourceStyleKeys.ToolStyle;
				if (rd.Contains(sKey))
					base.Style = (Style)rd[sKey];
			}
			this.SetAppearance();
			base.IsEnabled = _pTool.IsEnabled;
			base.IsChecked = false;

		}

		public Cursor MouseCursor
		{
			get
			{
				return _pTool.MouseCursor;
			}
		}

		public void Deactivate()
		{
			base.IsChecked = false;
			_pTool.Deactivate();
		}

		protected override void OnClick()
		{
			base.OnClick();
			base.IsChecked = true;
			this.Activated(this, new EventArgs());
			_pTool.Activate();
		}

		private void _pTool_IsEnabledChanged(object sender, EventArgs e)
		{
			base.IsEnabled = _pTool.IsEnabled;
		}

		private void pSplitButton_SelectionChanged(object sender, EventArgs e)
		{
			this.SetAppearance();
			this.Activated(this, new EventArgs());
			_pTool.Activate();
		}

		private void SetAppearance()
		{
			ResourceDictionary rd = MapApplication.Current.Resources;
			object obj = _pTool.ButtonContent;
			if ((rd != null) && (obj is Image))
			{
				Image img = (Image)obj;
				string sKey = ResourceStyleKeys.ButtonImageStyle;
				if (rd.Contains(sKey))
					img.Style = (Style)rd[sKey];
			}
			base.Content = obj;
			base.ToolTip = _pTool.ToolTip;
		}
	}

	public class MapDropButton : Button
	{

		private IMapSplitButton _pParent;

		public MapDropButton(IMapSplitButton pParent, ContextMenu MyContextMenu)
		{
			_pParent = pParent;
			if (_pParent is IMapButton)
			{
				IMapButton pButton = (IMapButton)_pParent;
				base.IsEnabled = pButton.IsEnabled;
				pButton.IsEnabledChanged += new EventHandler<EventArgs>(Parent_IsEnabledChanged);
			}
			else if (_pParent is IMapTool)
			{
				IMapTool pTool = (IMapTool)_pParent;
				base.IsEnabled = pTool.IsEnabled;
				pTool.IsEnabledChanged += new EventHandler<EventArgs>(Parent_IsEnabledChanged);
			}
			base.ContextMenu = MyContextMenu;
			ResourceDictionary rd = MapApplication.Current.Resources;
			if (rd != null)
			{
				string sKey = ResourceStyleKeys.DropButtonStyle;
				if (rd.Contains(sKey))
					base.Style = (Style)rd[sKey];
			}
			object obj = _pParent.DropButtonContent;
			if (obj is Image)
			{
				Image img = (Image)obj;
				string sKey = ResourceStyleKeys.DropButtonImageStyle;
				if (rd.Contains(sKey))
					img.Style = (Style)rd[sKey];
			}
			base.Content = obj;
			foreach (MenuItem menuItem in MyContextMenu.Items)
				menuItem.Click += new RoutedEventHandler(menuItem_Click);
		}

		protected override void OnClick()
		{
			base.OnClick();
			base.ContextMenu.IsOpen = true;
		}

		private void Parent_IsEnabledChanged(object sender, EventArgs e)
		{
			if (_pParent is IMapButton)
			{
				IMapButton pButton = (IMapButton)_pParent;
				base.IsEnabled = pButton.IsEnabled;
			}
			else if (_pParent is IMapTool)
			{
				IMapTool pTool = (IMapTool)_pParent;
				base.IsEnabled = pTool.IsEnabled;
			}
		}

		private void menuItem_Click(object sender, RoutedEventArgs e)
		{
			MapMenuItem item = (MapMenuItem)sender;
			IMapMenuItem pMenuItem = item.MenuItem;
			_pParent.OnItemSelected(pMenuItem);
		}

	}

	public class MapComboBox : ComboBox
	{

		private IMapComboBox _pComboBox;

		public MapComboBox(IMapComboBox pComboBox)
		{
			_pComboBox = pComboBox;
			_pComboBox.IsEnabledChanged += new EventHandler<EventArgs>(_pComboBox_IsEnabledChanged);
			_pComboBox.ItemsChanged += new EventHandler<EventArgs>(_pComboBox_ItemsChanged);
			base.IsEnabled = _pComboBox.IsEnabled;
			base.IsEditable = _pComboBox.IsEditable;
			base.IsReadOnly = _pComboBox.IsReadOnly;
			base.ItemsSource = _pComboBox.Items;
			base.SelectedIndex = 0;
		}

		protected override void OnSelectionChanged(SelectionChangedEventArgs e)
		{
			base.OnSelectionChanged(e);
			object oVal = base.SelectedValue;
			if (oVal == null)
				return;
			int i = base.SelectedIndex;
			_pComboBox.OnSelectionChanged(i);
		}

		protected override void OnKeyUp(KeyEventArgs e)
		{
			base.OnKeyUp(e);
			if (e.Key != Key.Enter)
				return;
			string sText = base.Text;
			_pComboBox.OnTextEntered(sText);
		}

		private void _pComboBox_IsEnabledChanged(object sender, EventArgs e)
		{
			base.IsEnabled = _pComboBox.IsEnabled;
		}

		void _pComboBox_ItemsChanged(object sender, EventArgs e)
		{
			base.SelectedIndex = 0;
		}

	}

}
