﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Media;

namespace FieldApp.Extensibility
{

	#region Basic Interaction
	public interface IStatus
	{
		void Status(string sMsg);
		void Status(string sMsg, bool bBusy);
		void Message(string sMsg);
		void Error(string sErr);
	}

	public interface IMainWindow
	{
		string ConfigurationFile { get; }
		void SetTitle(string sTitle);
		void SetLayout(UserControl ucLayout);
		void AddWindow(ParentWindow parentWindow);
		void Exit();
	}

	public interface IParentWindow
	{
		void SetTitle(string sTitle);
		void SetIcon(ImageSource srcIcon);
		void ShowWindow();
		void HideWindow();
	}

	public interface IDialogWindow
	{
		bool OpenDialog();
		void CloseDialog(bool bOK);
	}

	public interface IContent
	{
		// A user control in a ParentWindow should implement this if it needs to perform any cleanup
		void HideContent();
		void ShutDown();
	}

	public interface IExpandableContent
	{
		// This interface has no meaning in an ArcGIS Viewer for Silverlight addin
		// It is kept for WPF compatibility
		event EventHandler<EventArgs> ContentSizeChanged;
	}

	public interface IBehavior
	{
		// A Behavior must implement IBehavior
		void OnAttached();
	}

	public interface IConfigurable
	{
		void LoadConfiguration(string sConfigData);
		//string SaveConfiguration();
		//void Configure();
	}
	#endregion

	#region Menu and Toolbar Items
	public interface IMapMenuItem
	{
		Image Icon { get; }
		bool IsCheckable { get; }
		bool IsChecked { get; }
		bool IsEnabled { get; }
		string Label { get; }
		// After calling OnClick, if IsCheckable is true, 
		// FieldApp refreshes IsChecked
		void OnClick();  
		event EventHandler<EventArgs> IsEnabledChanged;
	}

	public interface IMapButton
	{
		object ButtonContent { get; } // Should return Image or string
		bool IsEnabled { get; }
		string ToolTip { get; }
		void OnClick();
		event EventHandler<EventArgs> IsEnabledChanged;
	}

	public interface IMapTool
	{
		object ButtonContent { get; } // Should return Image or string
		Cursor MouseCursor { get; }
		bool IsEnabled { get; }
		string ToolTip { get; }
		void Activate();
		void Deactivate();
		event EventHandler<EventArgs> IsEnabledChanged;
	}

	// A Button or Tool that is part of a SplitButton
	// must also implement this interface.
	public interface IMapSplitButton
	{
		object DropButtonContent { get; } // Should return Image or string
		void OnItemSelected(IMapMenuItem item);
		event EventHandler<EventArgs> SelectionChanged;
	}

	public interface IMapComboBox
	{
		bool IsEditable { get; }
		bool IsEnabled { get; }
		bool IsReadOnly { get; }
		ObservableCollection<object> Items { get; }
		void OnSelectionChanged(int iIndex);
		void OnTextEntered(string sNewText);
		event EventHandler<EventArgs> IsEnabledChanged;
		event EventHandler<EventArgs> ItemsChanged;
	}
	#endregion

}
