﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace FieldApp.Extensibility
{
	/// <summary>
	/// Interaction logic for ParentWindow.xaml
	/// </summary>
	public partial class ParentWindow : Window, IParentWindow, IDialogWindow
	{

		private UserControl _uc;
		private IContent _pContent;
		private bool _bResizable = false;
		private bool _bModal = false;
		private bool _bShutDown = false;

		public ParentWindow()
		{
			InitializeComponent();
			MapApplication.Current.AddWindow(this);
		}

		public void SetContent(UserControl uc, bool bResizable)
		{
			_uc = uc;
			LayoutRoot.Children.Add(uc);
			uc.VerticalAlignment = System.Windows.VerticalAlignment.Top;
			if (uc is IContent)
				_pContent = (IContent)uc;
			if (uc is IExpandableContent)
			{
				IExpandableContent pEC = (IExpandableContent)uc;
				pEC.ContentSizeChanged += pEC_ContentSizeChanged;
			}
			this.SizeToContent = System.Windows.SizeToContent.WidthAndHeight;
			_bResizable = bResizable;
			if (_bResizable)
				this.ResizeMode = System.Windows.ResizeMode.CanResizeWithGrip;
			else
				this.ResizeMode = System.Windows.ResizeMode.CanMinimize;
			
		}

		void pEC_ContentSizeChanged(object sender, EventArgs e)
		{
			// This forces a resize to content if a user has been resizing the window
			this.SizeToContent = System.Windows.SizeToContent.WidthAndHeight;
		}

		//void ParentWindow_SizeChanged(object sender, SizeChangedEventArgs e)
		//{
		//   if (e.PreviousSize == new Size(0, 0))
		//      return;
		//   if (e.NewSize == new Size(0, 0))
		//      return; // This should never happen?
		//   double dW = e.NewSize.Width - e.PreviousSize.Width;
		//   double dH = e.NewSize.Height - e.PreviousSize.Height;
		//   _uc.Width += dW;
		//   _uc.Height += dH;
		//}

		public void ShutDown()
		{
			if (_bShutDown)
				return;
			if (_pContent != null)
				_pContent.ShutDown();
			_bShutDown = true;
		}

		#region IParentWindow Implementation
		public void SetTitle(string sTitle)
		{
			Title = sTitle;
		}

		public void SetIcon(ImageSource srcIcon)
		{
			if (srcIcon == null)
			{
				WindowStyle = System.Windows.WindowStyle.ToolWindow;
				return;
			}
			Icon = srcIcon;
		}

		public void ShowWindow()
		{
			this.Show();
		}

		public void HideWindow()
		{
			if (_pContent != null)
				_pContent.HideContent();
			this.Hide();
		}
		#endregion

		#region IDialogWindow Implementation
		public bool OpenDialog()
		{
			_bModal = true;
			return (this.ShowDialog() == true);
		}

		public void CloseDialog(bool bOK)
		{
			this.DialogResult = bOK;
		}
		#endregion

		protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
		{
			base.OnClosing(e);
			if (_bModal)
			{
				this.ShutDown();
				return;
			}
			this.HideWindow();
			e.Cancel = true;
		}

	}
}
