﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.IsolatedStorage;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Toolkit;

using FieldApp.Extensibility;

namespace FieldApp
{

	public partial class MainWindow : Window, IMainWindow
	{

		public static string ConfigFile = "";

		private const string _sWindowSettings = "WindowSettings.txt";
		private MapApplication _MapApplication;
		private List<ParentWindow> _Windows = new List<ParentWindow>();

		public MainWindow()
		{
			// License setting and ArcGIS Runtime initialization is done in Application.xaml.cs.

			InitializeComponent();

			// Restore saved main window size and position

			this.Closing += new System.ComponentModel.CancelEventHandler(MainWindow_Closing);
			IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForAssembly();
			if (storage.FileExists(_sWindowSettings))
			{
				using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(_sWindowSettings, FileMode.Open, storage))
				{
					using (StreamReader reader = new StreamReader(stream))
					{
						string s = reader.ReadLine();
						Rect restoreBounds = Rect.Parse(s);
						if (!restoreBounds.IsEmpty)
						{
							this.Left = restoreBounds.Left;
							this.Top = restoreBounds.Top;
							this.Width = restoreBounds.Width;
							this.Height = restoreBounds.Height;
						}
					}
				}
			}

			// Initialize MapApplication
	
			_MapApplication = MapApplication.Current;

		}

		#region IMainWindow Implementation
		public string ConfigurationFile
		{
			get { return ConfigFile; }
		}

		public void SetTitle(string sTitle)
		{
			this.Title = sTitle;
		}

		public void SetLayout(UserControl ucLayout)
		{
			LayoutRoot.Children.Add(ucLayout);
		}

		public void AddWindow(ParentWindow parentWindow)
		{
			parentWindow.Owner = this;
			_Windows.Add(parentWindow);
		}

		public void Exit()
		{
			this.Close();
		}
		#endregion

		private void MainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{

			// Shut down windows

			foreach (ParentWindow PW in _Windows)
				PW.ShutDown();

			// Save window size and position

			IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForAssembly();
			using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(_sWindowSettings, FileMode.Create, storage))
			{
				using (StreamWriter writer = new StreamWriter(stream))
				{
					writer.WriteLine(this.RestoreBounds.ToString());
				}
			}

		}

	}
}
