﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows;

namespace FieldAppUpdater
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{

		public static string Configuration = "";
		private const string RegBase = @"HKEY_LOCAL_MACHINE\Software\UNS\FieldApp";

		public MainWindow()
		{
			InitializeComponent();
			btnCancel.Click += new RoutedEventHandler(btnCancel_Click);
			btnOK.Click += new RoutedEventHandler(btnOK_Click);
		}

		void btnCancel_Click(object sender, RoutedEventArgs e)
		{
			this.Close();
		}

		void btnOK_Click(object sender, RoutedEventArgs e)
		{

			// Get update path

			string sUpdatePath = "";
			object oVal = Registry.GetValue(RegBase, "UpdatePath", null);
			if (oVal == null)
			{
				string sMsg = "Update path not found in registry.";
				MessageBoxResult mbResult = MessageBox.Show(sMsg, "FieldApp Updater", MessageBoxButton.OK);
				return;
			}
			sUpdatePath = oVal.ToString();
			if (!Directory.Exists(sUpdatePath))
				return;

			// Check for updates

			Assembly a = Assembly.GetExecutingAssembly();
			string sLocalPath = Path.GetDirectoryName(a.Location);
			string sRemotePath = sUpdatePath + "\\FieldApp";
			foreach (string sRemoteFileName in Directory.GetFiles(sRemotePath))
			{
				string sLocalFileName = sRemoteFileName.Replace(sRemotePath, sLocalPath);
				if (sLocalFileName.ToLower().Contains("fieldappupdater.exe"))
					continue;
				this.CheckForUpdate(sLocalFileName, sRemoteFileName, false);
			}

			// Restart FieldApp

			string sFieldApp = sLocalPath + "\\" + "FieldApp.exe";
			if (Configuration != "")
				System.Diagnostics.Process.Start(sFieldApp, Configuration);
			else
				System.Diagnostics.Process.Start(sFieldApp);
			Application.Current.Shutdown();

		}

		private bool CheckForUpdate(string sLocalFileName, string sUpdateFileName, bool bCheckOnly)
		{

			bool bUpdate = false;
			if (!File.Exists(sLocalFileName))
				bUpdate = true; // This should never happen
			else
			{
				FileInfo fiUpdate = new FileInfo(sUpdateFileName);
				FileInfo fiLocal = new FileInfo(sLocalFileName);
				if (fiUpdate.LastWriteTime > fiLocal.LastWriteTime)
					bUpdate = true;
			}
			if (bCheckOnly)
				return bUpdate;
			if (!bUpdate)
				return bUpdate;
			try
			{
				File.Copy(sUpdateFileName, sLocalFileName, true);
			}
			catch (Exception ex)
			{
				string sErr = "Error updating " + sLocalFileName + ":\n" + ex.Message;
				MessageBox.Show(sErr, "FieldApp Updater", MessageBoxButton.OK, MessageBoxImage.Error);
				bUpdate = false;
			}
			return bUpdate;

		}


	}
}
